<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

use Util\DateUtil;

class AdmissaoRepository {
    /** @var EntityManager */
    protected $entityManager;

    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->entityManager        = $entityManager;
        $this->connection           = $entityManager->getConnection();
    }

    public function getVinculosNaoAprovados(array $params): array {
        $sql = <<<SQL
            SELECT /*+ ordered*/
                VM.*,
                PH.*,
                PH.CDPAIS AS CDPAISNATURALI,
                PH.SGESTADO AS SGESTADONATURALI,
                PH.CDMUNICIPIO AS CDMUNICIPIONATURALI,
                VH.NRVINCULOH,
                VH.NRSITUFUNCM,
                VH.NRCARGO,
                VH.NRESCALATRABM,
                VH.DTMESCOMPETENC,
                VH.IDCONTRIBUISIND,
                VH.NRDEPENDIR,
                VH.NRDEPENDSFAM,
                VH.DTBASEFERIAS,
                VH.IDREMUNERACAO,
                VH.IDTPPAGAMENTO,
                VH.NRESTRUTLEGAL,
                VH.NRESTRUTGEREN,
                VH.NRTPMODALIDSAL,
                VH.IDMULTVINC,
                VH.NRFUNCAO,
                VH.CDCONTRIBINDIVIDUAL,
                VH.NRTPMOVTRANSFM,
                VH.DTFIMCONTRDETERMIN,
                VH.NRESTRUTSIND,
                VH.CONFIDENCIAL,
                VH.NRTURNO,
                VH.NRTPREGIMEPREV,
                NVL(VH.IDPAGPROPFER13, 'N') IDPAGPROPFER13,
                FPA_TPADMISSAO.NMTPADMISSAO,
                ESTRUTLEGAL.NMESTRUTURAM AS NMESTRUTLEGAL,
                ESTRUTGEREN.NMESTRUTURAM AS NMESTRUTGEREN,
                ESTRUTSIND.NMESTRUTURAM AS NMESTRUTSIND,
                TVH.NMTIPOVINCULOH,
                VE.NMVINCULOEMPREG,
                VE.NMABREV,
                TMT.NMTPMOVTRANSFH,
                RP.DSTPREGIMEPREV,
                CARGO.NMOCUPACAOH AS NMCARGO,
                FUNCAO.NMOCUPACAOH AS NMFUNCAO,
                CBO.NRCBO,
                CBO.NMCBO,
                GPE_ALTEESCALA.DTINIESCALA,
                GPE_ALTEESCALA.DTFIMESCALA,
                GPE_ALTEESCALA.QTHRLIMITESEPHEXTRA,
                ESC.NMESCALATRABH,
                GPE_TURNO.DSTURNO,
                SH.NMSITUFUNCH,
                TMS.NMTPMODALIDSAL,
                GPE_ALTESALARIO.IDTPSALARIO,
                GPE_ALTESALARIO.VRSALARIO,
                GPE_ALTESALARIO.NROCORRENCIA,
                GPE_TIPOSALARIO.DSTIPOSALARIO,
                ESTRG.NMESTADO AS NMUFRGPESSOA,
                ESTCTPS.NMESTADO AS NMUFCTPSPES,
                CONTCORRPARC.NRCONTCORRPARC,
                CONTCORRPARC.CDBANCO,
                CONTCORRPARC.CDAGENCIA,
                CONTCORRPARC.CDCONTCORR,
                CONTCORRPARC.NRTPCONTCORR,
                BANCO.NMBANCO,
                AGENCIA.NMAGENCIA,
                TPCONTCORR.DSTPCONTCORR,
                TELPRINCIPAL_.CDPREFIXCOMUPARC PREFTELPRINCIPAL,
                TELPRINCIPAL_.CDCOMUNICAPARC TELPRINCIPAL,
                TELRESIDEN_.CDPREFIXCOMUPARC PREFTELRESIDEN,
                TELRESIDEN_.CDCOMUNICAPARC TELRESIDEN,
                TELCOMERC_.CDPREFIXCOMUPARC PREFTELCOMERC,
                TELCOMERC_.CDCOMUNICAPARC TELCOMERC,
                CELPARTIC_.CDPREFIXCOMUPARC PREFCELPARTIC,
                CELPARTIC_.CDCOMUNICAPARC CELPARTIC,
                CELCOMERC_.CDPREFIXCOMUPARC PREFCELCOMERC,
                CELCOMERC_.CDCOMUNICAPARC CELCOMERC,
                FAX_.CDPREFIXCOMUPARC PREFFAX,
                FAX_.CDCOMUNICAPARC FAX,
                EMAIL_.CDCOMUNICAPARC EMAIL,
                ENDE.NRENDERECOPARC,
                ENDE.CDLOGRADOURO,
                ENDE.DSENDERECO,
                ENDE.NRIMOVELENDERECO,
                ENDE.NMBAIRROENDERECO,
                ENDE.NRCEPENDERECO,
                ENDE.SGESTADO,
                ENDE.CDPAIS,
                ENDE.CDMUNICIPIO,
                ENDE.DSCOMPLEENDERECO,
                ENDE.DSREFERENCIAENDE,
                ENDE.CDTIPOENDERECO,
                MUN.NMMUNICIPIO,
                PAIS.NMPAIS,
                CLEST.DSCLASSESTRANG
            FROM
                GPE_PESSOA PE
                JOIN
                GPE_PESSOAH PH ON
                    PH.NRPESSOA = PE.NRPESSOA AND
                    PH.NRORG =  PE.NRORG
                LEFT JOIN
                GPE_VINCULOM VM ON
                    PE.NRPESSOA = VM.NRPESSOA AND
                    PE.NRORG = VM.NRORG
                LEFT JOIN
                GPE_VINCULOH VH ON
                    VH.NRVINCULOM = VM.NRVINCULOM AND
                    VH.NRORG = VM.NRORG
                LEFT JOIN
                GPE_TIPOVINCULOH TVH ON
                    TVH.NRTIPOVINCULOM = VM.NRTIPOVINCULOM AND
                    (
                        TVH.NRORG = VM.NRORG OR
                        TVH.NRORG =:P_NRORG_PADRAO
                    ) AND
                    TVH.DTMESCOMPETENC = (
                        SELECT
                            MAX(TVHH.DTMESCOMPETENC)
                        FROM
                            GPE_TIPOVINCULOH TVHH
                        WHERE
                            TVHH.NRORG = TVH.NRORG AND
                            TVHH.NRTIPOVINCULOM = TVH.NRTIPOVINCULOM AND
                            TVHH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_TPMOVTRANSFH TMT ON
                    TMT.NRTPMOVTRANSFM = VH.NRTPMOVTRANSFM AND
                    (
                        TMT.NRORG = VH.NRORG OR
                        TMT.NRORG =:P_NRORG_PADRAO
                    ) AND
                    TMT.DTMESCOMPETENC = (
                        SELECT
                            MAX(TMTH.DTMESCOMPETENC)
                        FROM
                            GPE_TPMOVTRANSFH TMTH
                        WHERE
                            TMTH.NRORG = TMT.NRORG AND
                            TMTH.NRTPMOVTRANSFM = TMT.NRTPMOVTRANSFM AND
                            TMTH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_OCUPACAOH CARGO ON
                    CARGO.NROCUPACAOM = VH.NRCARGO AND
                    CARGO.NRORG = VH.NRORG AND
                    CARGO.DTMESCOMPETENC = (
                        SELECT
                            MAX(CARGOH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH CARGOH
                        WHERE
                            CARGOH.NRORG = CARGO.NRORG AND
                            CARGOH.NROCUPACAOM = CARGO.NROCUPACAOM AND
                            CARGOH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_OCUPACAOH FUNCAO ON
                    FUNCAO.NROCUPACAOM = VH.NRFUNCAO AND
                    FUNCAO.NRORG = VH.NRORG AND
                    FUNCAO.DTMESCOMPETENC = (
                        SELECT
                            MAX(FUNCAOH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH FUNCAOH
                        WHERE
                            FUNCAOH.NRORG = FUNCAO.NRORG AND
                            FUNCAOH.NROCUPACAOM = FUNCAO.NROCUPACAOM AND
                            FUNCAOH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                GPE_CBO CBO ON
                    FUNCAO.NRCBO = CBO.NRCBO
                LEFT JOIN
                ESTRUTURAM ESTRUTLEGAL ON
                    ESTRUTLEGAL.NRESTRUTURAM = VH.NRESTRUTLEGAL AND
                    ESTRUTLEGAL.NRORG = VH.NRORG
                LEFT JOIN
                ESTRUTURAM ESTRUTGEREN ON
                    ESTRUTGEREN.NRESTRUTURAM = VH.NRESTRUTGEREN AND
                    ESTRUTGEREN.NRORG = VH.NRORG
                LEFT JOIN
                ESTRUTURAM ESTRUTSIND ON
                    ESTRUTSIND.NRESTRUTURAM = VH.NRESTRUTSIND AND
                    ESTRUTSIND.NRORG = VH.NRORG
                LEFT JOIN
                ESTADO ESTRG ON
                    ESTRG.SGESTADO = PH.SGUFRGPESSOA AND
                    ESTRG.CDPAIS = '0055'
                LEFT JOIN
                ESTADO ESTCTPS ON
                    ESTCTPS.SGESTADO = PH.SGUFCTPSPES AND
                    ESTCTPS.CDPAIS = '0055'
                LEFT JOIN
                GPE_VINCULOEMPREG VE ON
                    VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
                LEFT JOIN
                FPA_TPMODALIDSAL TMS ON
                    TMS.NRTPMODALIDSAL = VH.NRTPMODALIDSAL AND
                    (
                        VH.NRORG = TMS.NRORG OR
                        TMS.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_ALTESALARIO ON
                    VM.NRVINCULOM = GPE_ALTESALARIO.NRVINCULOM AND
                    VM.NRORG = GPE_ALTESALARIO.NRORG
                LEFT JOIN
                GPE_TIPOSALARIO ON
                    GPE_TIPOSALARIO.CDTIPOSALARIO = GPE_ALTESALARIO.IDTPSALARIO
                LEFT JOIN
                GPE_TPREGIMEPREV RP ON
                    RP.NRTPREGIMEPREV = VH.NRTPREGIMEPREV AND
                    (
                        VH.NRORG = RP.NRORG OR
                        RP.NRORG =:P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_ALTEESCALA ON
                    GPE_ALTEESCALA.NRESCALATRABM = VH.NRESCALATRABM AND
                    GPE_ALTEESCALA.NRVINCULOM = VM.NRVINCULOM AND
                    GPE_ALTEESCALA.NRORG = VH.NRORG
                LEFT JOIN
                GPE_ESCALATRABH ESC ON
                    ESC.NRESCALATRABM = GPE_ALTEESCALA.NRESCALATRABM AND
                    ESC.NRORG = VH.NRORG AND
                    ESC.DTMESCOMPETENC = (
                        SELECT
                            MAX(ESCH.DTMESCOMPETENC)
                        FROM
                            GPE_ESCALATRABH ESCH
                        WHERE
                            ESCH.NRORG = ESC.NRORG AND
                            ESCH.NRESCALATRABM = ESC.NRESCALATRABM AND
                            ESCH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
                LEFT JOIN
                GPE_TURNO ON
                    GPE_TURNO.NRTURNO = GPE_ALTEESCALA.NRTURNO AND
                    GPE_TURNO.NRORG = GPE_ALTEESCALA.NRORG
                LEFT JOIN
                GPE_SITUFUNCH SH ON
                    (
                        SH.NRORG = VH.NRORG OR
                        SH.NRORG =:P_NRORG_PADRAO
                    ) AND
                    SH.NRSITUFUNCM = VH.NRSITUFUNCM AND
                    SH.DTEFETIVA = (
                        SELECT
                            MAX(SHH.DTEFETIVA)
                        FROM
                            GPE_SITUFUNCH SHH
                        WHERE
                            SHH.NRORG = SH.NRORG AND
                            SHH.NRSITUFUNCM = SH.NRSITUFUNCM AND
                            DTEFETIVA <=:P_DTMESCOMPETENC
                    )
                LEFT JOIN
                CONTCORRPARC ON
                        CONTCORRPARC.NRPARCNEGOCIO = PE.NRPARCNEGOCIO AND
                        CONTCORRPARC.NRORG = PE.NRORG
                LEFT JOIN
                BANCO ON
                    BANCO.CDBANCO = CONTCORRPARC.CDBANCO AND
                    (
                        BANCO.NRORG = CONTCORRPARC.NRORG OR
                        BANCO.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                TPCONTCORR ON
                    TPCONTCORR.NRTPCONTCORR = CONTCORRPARC.NRTPCONTCORR AND
                    (
                        TPCONTCORR.NRORG = CONTCORRPARC.NRORG OR
                        TPCONTCORR.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                AGENCIA ON
                    CONTCORRPARC.CDAGENCIA = AGENCIA.CDAGENCIA AND
                    CONTCORRPARC.CDBANCO = AGENCIA.CDBANCO AND
                    (
                        AGENCIA.NRORG = CONTCORRPARC.NRORG OR
                        AGENCIA.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                COMUNICAPARC TELPRINCIPAL_ ON
                    PE.NRPARCNEGOCIO = TELPRINCIPAL_.NRPARCNEGOCIO AND
                    PE.NRORG = TELPRINCIPAL_.NRORG AND
                    TELPRINCIPAL_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELPRINCIPAL_.NRPARCNEGOCIO AND
                            CP.NRORG = TELPRINCIPAL_.NRORG AND
                            CP.CDFORMACOMU = '01'
                    )
                LEFT JOIN
                COMUNICAPARC TELRESIDEN_ ON
                    PE.NRPARCNEGOCIO = TELRESIDEN_.NRPARCNEGOCIO AND
                    PE.NRORG = TELRESIDEN_.NRORG AND
                    TELRESIDEN_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELRESIDEN_.NRPARCNEGOCIO AND
                            CP.NRORG = TELRESIDEN_.NRORG AND
                            CP.CDFORMACOMU = '03'
                        )
                LEFT JOIN
                COMUNICAPARC TELCOMERC_ ON
                    PE.NRPARCNEGOCIO = TELCOMERC_.NRPARCNEGOCIO AND
                    PE.NRORG = TELCOMERC_.NRORG AND
                    TELCOMERC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = TELCOMERC_.NRPARCNEGOCIO AND
                            CP.NRORG = TELCOMERC_.NRORG AND
                            CP.CDFORMACOMU = '04'
                        )
                LEFT JOIN
                COMUNICAPARC CELPARTIC_ ON
                    PE.NRPARCNEGOCIO = CELPARTIC_.NRPARCNEGOCIO AND
                    PE.NRORG = CELPARTIC_.NRORG AND
                    CELPARTIC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = CELPARTIC_.NRPARCNEGOCIO AND
                            CP.NRORG = CELPARTIC_.NRORG AND
                            CP.CDFORMACOMU = '02'
                        )
                LEFT JOIN
                COMUNICAPARC CELCOMERC_ ON
                    PE.NRPARCNEGOCIO = CELCOMERC_.NRPARCNEGOCIO AND
                    PE.NRORG = CELCOMERC_.NRORG AND
                    CELCOMERC_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = CELCOMERC_.NRPARCNEGOCIO AND
                            CP.NRORG = CELCOMERC_.NRORG AND
                            CP.CDFORMACOMU = '06'
                        )
                LEFT JOIN
                COMUNICAPARC FAX_ ON
                    PE.NRPARCNEGOCIO = FAX_.NRPARCNEGOCIO AND
                    PE.NRORG = FAX_.NRORG AND
                    FAX_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = FAX_.NRPARCNEGOCIO AND
                            CP.NRORG = FAX_.NRORG AND
                            CP.CDFORMACOMU = '10'
                        )
                LEFT JOIN
                COMUNICAPARC EMAIL_ ON
                    PE.NRPARCNEGOCIO = EMAIL_.NRPARCNEGOCIO AND
                    PE.NRORG = EMAIL_.NRORG AND
                    EMAIL_.NRCOMUNICAPARC = (
                        SELECT
                            MAX(CP.NRCOMUNICAPARC)
                        FROM
                            COMUNICAPARC CP
                        WHERE
                            CP.NRPARCNEGOCIO = EMAIL_.NRPARCNEGOCIO AND
                            CP.NRORG = EMAIL_.NRORG AND
                            CP.CDFORMACOMU = '09'
                        )
                LEFT JOIN
                ENDERECOPARC ENDE ON
                    PE.NRPARCNEGOCIO = ENDE.NRPARCNEGOCIO AND
                    PE.NRORG = ENDE.NRORG AND
                    ENDE.NRENDERECOPARC = (
                        SELECT
                            MAX(ENDER.NRENDERECOPARC)
                        FROM
                            ENDERECOPARC ENDER
                        WHERE
                            ENDER.NRPARCNEGOCIO = ENDE.NRPARCNEGOCIO AND
                            ENDER.NRORG = ENDE.NRORG AND
                            ENDER.CDTIPOENDERECO = 'PRINCIPAL'
                        )
                LEFT JOIN
                PAIS ON
                    PAIS.CDPAIS = ENDE.CDPAIS
                LEFT JOIN
                MUNICIPIO MUN ON
                    MUN.CDMUNICIPIO = ENDE.CDMUNICIPIO AND
                    MUN.CDPAIS = ENDE.CDPAIS AND
                    ENDE.SGESTADO = MUN.SGESTADO
                LEFT JOIN
                FPA_TPADMISSAO ON
                    FPA_TPADMISSAO.NRTPADMISSAO = VM.NRTPADMISSAO AND
                    (
                        FPA_TPADMISSAO.NRORG = VM.NRORG OR
                        FPA_TPADMISSAO.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_CLASSESTRANG CLEST ON
                    CLEST.NRCLASSESTRANG = PH.NRCLASSESTRANG
            WHERE
                PE.NRORG =:P_NRORG AND
                (
                    VM.DTADMISSAOVINC <= LAST_DAY(:P_DTMESCOMPETENC) OR
                    VM.NRVINCULOM IS NULL
                ) AND
                (
                    VM.DTRESCISAOVINC IS NULL OR
                    VM.DTRESCISAOVINC >=:P_DTMESCOMPETENC
                ) AND
                (
                    (INSTR(:P_NRESTRUTURAS, ',' || VH.NRESTRUTGEREN || ',')) > 0 OR
                    :P_NRESTRUTURAS IS NULL
                ) AND (
                    (
                        TO_CHAR(PH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                        VM.NRVINCULOM IS NULL
                    ) OR
                    (
                        TO_CHAR(VH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                        VM.NRVINCULOM IS NOT NULL
                    )
                ) AND
                (
                    (
                        PH.DTMESCOMPETENC = (
                            SELECT
                                MAX(PHH.DTMESCOMPETENC)
                            FROM GPE_PESSOAH PHH
                            WHERE
                                PHH.NRORG = PH.NRORG AND
                                PHH.NRPESSOA = PH.NRPESSOA AND
                                PHH.DTMESCOMPETENC <=:P_DTMESCOMPETENC
                        ) AND
                        VM.NRVINCULOM IS NOT NULL
                    ) OR
                    TO_CHAR(PH.DTMESCOMPETENC, 'DD/MM/YYYY') = '01/01/3000' AND
                    VM.NRVINCULOM IS NULL
                )
            ORDER BY PH.NMPESSOA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getTipoFormacaoParc(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                F.*
            FROM
                DES_TIPOFORMACAO F
            WHERE
                F.NRORG = :P_NRORG OR
                F.NRORG = :P_NRORG_PADRAO
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tiposFormacao = $std->fetchAll();

        return $tiposFormacao;
    }

    public function getCursoAcademico(): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                DES_CURSO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute();

        $cursos = $std->fetchAll();

        return $cursos;
    }

    public function getInstituicaoEstrutura(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                PARCNEGOCIO
            WHERE
                NRORG IN (:P_NRORG, :P_NRORG_PADRAO) AND
                CDTIPOPARCPRINCIPAL = 'ESTRUTURA'
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $instituicoes = $std->fetchAll();

        return $instituicoes;
    }

    public function getAreaFormacao(): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                DES_AREAFORMACAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute();

        $areas = $std->fetchAll();

        return $areas;
    }

    public function getBeneficio(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                BNM.DTINICIOVIGENCIA AS DTINICIOVIGENCIA,
                BNM.DTFINALVIGENCIA AS DTFINALVIGENCIA,
                BNH.VRTITULARSIND AS VRTITULARSIND,
                BNH.VRDEPSIND AS VRDEPSIND,
                BNH.NRORG AS NRORG,
                BNH.CONSVINCRESMESANT AS CONSVINCRESMESANT,
                BNH.NRBENEFICIOM AS NRBENEFICIOM,
                BNH.NRBENEFICIOH AS NRBENEFICIOH,
                BNH.NRTPBENEFICIO AS NRTPBENEFICIO,
                BNH.NRFORMABENEFICIO AS NRFORMABENEFICIO,
                BNH.NRESTRUTURAM AS NRESTRUTURAM,
                BNH.DTMESCOMPETENC AS DTMESCOMPETENC,
                BNH.DTREFERENCIA AS DTREFERENCIA,
                BNH.DSBENEFICIO AS DSBENEFICIO,
                BNH.TPPERIODOBENEF AS TPPERIODOBENEF,
                BNH.DSOBSBENEFICIO AS DSOBSBENEFICIO,
                BNH.NRTABELAFOLHAPRI AS NRTABELAFOLHAPRI,
                BNH.NRTABELAFOLHASEC AS NRTABELAFOLHASEC,
                BNH.IDATIVO AS IDATIVO,
                BNM.NRORGINCLUSAO AS NRORGINCLUSAO,
                BNM.DTINCLUSAO AS DTINCLUSAO,
                BNM.CDOPERINCLUSAO AS CDOPERINCLUSAO,
                BNH.NRORGULTATU AS NRORGULTATU,
                BNH.DTULTATU AS DTULTATU,
                BNH.CDOPERULTATU AS CDOPERULTATU,
                BNH.NREVENTOMENSA AS NREVENTOMENSA,
                BNH.NREVENTOMENSADEP AS NREVENTOMENSADEP,
                BNH.NREVETITULAREMP AS NREVETITULAREMP,
                BNH.NREVEDEPEMPRESA AS NREVEDEPEMPRESA,
                BNH.NREVETITULARVINC AS NREVETITULARVINC,
                BNH.NREVEDEPEVINC AS NREVEDEPEVINC,
                BNH.NREVENTOCOPART AS NREVENTOCOPART,
                BNH.NREVENTOCOPARTDEP AS NREVENTOCOPARTDEP,
                BNH.NREVENTOTAXA AS NREVENTOTAXA,
                BNH.NRTABFAIXASALTIT AS NRTABFAIXASALTIT,
                BNH.NRTABFAIXASALDEP AS NRTABFAIXASALDEP,
                BNH.NRTIPOPARTICIPATITULAR AS NRTIPOPARTICIPATITULAR,
                BNH.NRTIPOPARTICIPADEP AS NRTIPOPARTICIPADEP,
                BNH.NRMESANIVERSARIO AS NRMESANIVERSARIO,
                BNH.NRDIAVENCBOLETO AS NRDIAVENCBOLETO,
                BNH.VRLINEARBENEFICIO AS VRLINEARBENEFICIO,
                BNH.VRLINEARBENEFICIODEP AS VRLINEARBENEFICIODEP,
                BNH.VRPISOTITULARMENS AS VRPISOTITULARMENS,
                BNH.VRTETOTITULARMENS AS VRTETOTITULARMENS,
                BNH.NRIDADEMAXSUBFILHOS AS NRIDADEMAXSUBFILHOS,
                BNH.NRIDADEMAXSUBDEP AS NRIDADEMAXSUBDEP,
                BNH.VRPARTICIPATITULAR AS VRPARTICIPATITULAR,
                BNH.VRPARTICIPADEPE AS VRPARTICIPADEPE,
                BNH.VRPISOTITULAR AS VRPISOTITULAR,
                BNH.VRTETOTITULAR AS VRTETOTITULAR,
                BNH.VRPISODEP AS VRPISODEP,
                BNH.VRTETODEP AS VRTETODEP,
                BNH.VRPISOTITULARSIND AS VRPISOTITULARSIND,
                BNH.VRTETOTITULARSIND AS VRTETOTITULARSIND,
                BNH.VRPISODEPSIND AS VRPISODEPSIND,
                BNH.VRTETODEPSIND AS VRTETODEPSIND,
                BNH.SEXOSUBSIDIOFILHO AS SEXOSUBSIDIOFILHO,
                BNH.SEXOSUBSIDIODEP AS SEXOSUBSIDIODEP,
                BNH.IDCONSFALTABENEF,
                BNH.QTMESESFALTABENEF,
                BNH.IDCONSATESTBENEF,
                BNH.QTMESESATESTBENEF,
                BNH.IDCONSAFASTBENEF,
                BNH.QTMESESAFASTBENEF,
                BNH.IDCONSFERIASBENEF,
                BNH.QTMESESFERIASBENEF,
                BNH.IDCONSFERIADOBENEF,
                BNH.QTMESESFERIADOBENEF,
                BNH.IDCONSDSRBENEF,
                BNH.QTMESESDSRBENEF,
                BNH.IDGERABENEFFALTA,
                BNH.IDGERABENEFATEST,
                BNH.IDGERABENEFAFAST,
                BNH.IDGERADESCAFAST,
                BNH.IDGERABENEFFERIAS,
                BNH.IDPROPDTINSCR,
                BNH.IDPROPDTCANC,
                BNH.IDDUPLICMENS,
                BNH.NRDIARESCISAO,
                BNH.VREVENTOTAXA,
                BNH.NRESTRUTSIND,
                BNH.DTDISPONIB,
                BNH.IDPROPADMIS,
                BNH.IDPROPRESC,
                BNH.PERCOPARTICIPATITU AS PERCOPARTICIPATITU,
                BNH.PERCOPARTICIPADEPE AS PERCOPARTICIPADEPE,
                TPB.DSTPBENEFICIO,
                FB.DSFORMABENEFICIO,
                EST.NMESTRUTURAH,
                ESTSIND.NMESTRUTURAH AS NMESTRUTSIND,
                (
                    SELECT
                        LISTAGG(TFP.NMTABELAFOLHA, ', ') WITHIN GROUP (
                    ORDER BY
                        TFP.NMTABELAFOLHA)
                    FROM
                        FPA_TABELAFOLHA TFP
                    WHERE
                        INSTR( ', ' || BNH.NRTABELAFOLHAPRI || ', ', ', ' || TFP.NRTABELAFOLHA || ', ') > 0
                        AND
                        (
                            TFP.NRORG = BNM.NRORG
                            OR TFP.NRORG = :NRORG_PADRAO
                        )
                )
                NMTABELAFOLHAPRI,
                (
                    SELECT
                        LISTAGG(TFS.NMTABELAFOLHA, ', ') WITHIN GROUP (
                    ORDER BY
                        TFS.NMTABELAFOLHA)
                    FROM
                        FPA_TABELAFOLHA TFS
                    WHERE
                        INSTR( ', ' || BNH.NRTABELAFOLHASEC || ', ', ', ' || TFS.NRTABELAFOLHA || ', ') > 0
                        AND
                        (
                            TFS.NRORG = BNM.NRORG
                            OR TFS.NRORG = :NRORG_PADRAO
                        )
                )
                NMTABELAFOLHASEC,
                EVH_MENS.NMEVENTOH AS NMEVENTOMENSA,
                EVH_MENS_DEP.NMEVENTOH AS NMEVENTOMENSADEP,
                EVH_EMP_TIT.NMEVENTOH AS NMEVETITULAREMP,
                EVH_EMP_DEP.NMEVENTOH AS NMEVEDEPEMPRESA,
                EVH_TIT_VINC.NMEVENTOH AS NMEVETITULARVINC,
                EVH_DEP_VINC.NMEVENTOH AS NMEVEDEPEVINC,
                EVH_TIT_COPAR.NMEVENTOH AS NMEVENTOCOPART,
                EVH_DEP_COPAR.NMEVENTOH AS NMEVENTOCOPARTDEP,
                EVH_TAXA_INCL_IMPL.NMEVENTOH AS NMEVENTOTAXA,
                (
                    SELECT
                        LISTAGG(TF_FAIXSAL_TIT.NMTABELAFOLHA, ', ') WITHIN GROUP (
                    ORDER BY
                        TF_FAIXSAL_TIT.NMTABELAFOLHA)
                    FROM
                        FPA_TABELAFOLHA TF_FAIXSAL_TIT
                    WHERE
                        INSTR( ', ' || BNH.NRTABFAIXASALTIT || ', ', ', ' || TF_FAIXSAL_TIT.NRTABELAFOLHA || ', ') > 0
                        AND
                        (
                            TF_FAIXSAL_TIT.NRORG = BNM.NRORG
                            OR TF_FAIXSAL_TIT.NRORG = :NRORG_PADRAO
                        )
                )
                NMTABFAIXASALTIT,
                (
                    SELECT
                        LISTAGG(TF_FAIXSAL_DEP.NMTABELAFOLHA, ', ') WITHIN GROUP (
                    ORDER BY
                        TF_FAIXSAL_DEP.NMTABELAFOLHA)
                    FROM
                        FPA_TABELAFOLHA TF_FAIXSAL_DEP
                    WHERE
                        INSTR( ', ' || BNH.NRTABFAIXASALDEP || ', ', ', ' || TF_FAIXSAL_DEP.NRTABELAFOLHA || ', ') > 0
                        AND
                        (
                            TF_FAIXSAL_DEP.NRORG = BNM.NRORG
                            OR TF_FAIXSAL_DEP.NRORG = :NRORG_PADRAO
                        )
                )
                NMTABFAIXASALDEP,
                BNH.IDGERBENEFADMISSAO,
                BNH.PERIODICIDADEMES,
                SUBSTR(BNH.PERIODICIDADEMES, 1, 1) PERIODICIDADEMES01,
                SUBSTR(BNH.PERIODICIDADEMES, 2, 1) PERIODICIDADEMES02,
                SUBSTR(BNH.PERIODICIDADEMES, 3, 1) PERIODICIDADEMES03,
                SUBSTR(BNH.PERIODICIDADEMES, 4, 1) PERIODICIDADEMES04,
                SUBSTR(BNH.PERIODICIDADEMES, 5, 1) PERIODICIDADEMES05,
                SUBSTR(BNH.PERIODICIDADEMES, 6, 1) PERIODICIDADEMES06,
                SUBSTR(BNH.PERIODICIDADEMES, 7, 1) PERIODICIDADEMES07,
                SUBSTR(BNH.PERIODICIDADEMES, 8, 1) PERIODICIDADEMES08,
                SUBSTR(BNH.PERIODICIDADEMES, 9, 1) PERIODICIDADEMES09,
                SUBSTR(BNH.PERIODICIDADEMES, 10, 1) PERIODICIDADEMES10,
                SUBSTR(BNH.PERIODICIDADEMES, 11, 1) PERIODICIDADEMES11,
                SUBSTR(BNH.PERIODICIDADEMES, 12, 1) PERIODICIDADEMES12,
                BNH.IDGERBENEFRESCISAO
            FROM
                VAB_BENEFICIOM BNM
                LEFT JOIN
                    VAB_BENEFICIOH BNH
                    ON BNH.NRBENEFICIOM = BNM.NRBENEFICIOM
                    AND BNM.NRORG = BNH.NRORG
                    AND BNH.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(BNEH.DTMESCOMPETENC)
                        FROM
                            VAB_BENEFICIOH BNEH
                        WHERE
                            BNEH.NRORG = BNH.NRORG
                            AND BNEH.NRBENEFICIOM = BNH.NRBENEFICIOM
                            AND BNEH.DTMESCOMPETENC <= :DTMESCOMPETENC
                    )
                JOIN
                    TPBENEFICIO TPB
                    ON TPB.NRTPBENEFICIO = BNH.NRTPBENEFICIO
                    AND
                    (
                        TPB.NRORG = BNH.NRORG
                        OR TPB.NRORG = :NRORG_PADRAO
                    )
                JOIN
                    FORMABENEFICIO FB
                    ON FB.NRFORMABENEFICIO = BNH.NRFORMABENEFICIO
                    AND
                    (
                        FB.NRORG = BNH.NRORG
                        OR FB.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    ESTRUTURAH EST
                    ON EST.NRESTRUTURAM = BNH.NRESTRUTURAM
                    AND EST.NRORG = BNH.NRORG
                    AND EST.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(ESTH.DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH ESTH
                        WHERE
                            ESTH.NRORG = EST.NRORG
                            AND ESTH.NRESTRUTURAM = EST.NRESTRUTURAM
                            AND ESTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                    )
                LEFT JOIN
                    ESTRUTURAH ESTSIND
                    ON ESTSIND.NRESTRUTURAM = BNH.NRESTRUTSIND
                    AND ESTSIND.NRORG = BNH.NRORG
                    AND ESTSIND.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(ESTH.DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH ESTH
                        WHERE
                            ESTH.NRORG = ESTSIND.NRORG
                            AND ESTH.NRESTRUTURAM = ESTSIND.NRESTRUTURAM
                            AND ESTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_MENS
                    ON EVH_MENS.NREVENTOM = BNH.NREVENTOMENSA
                    AND
                    (
                        EVH_MENS.NRORG = BNM.NRORG
                        OR EVH_MENS.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_MENS.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_MENS.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_MENS.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_MENS_DEP
                    ON EVH_MENS_DEP.NREVENTOM = BNH.NREVENTOMENSADEP
                    AND
                    (
                        EVH_MENS_DEP.NRORG = BNM.NRORG
                        OR EVH_MENS_DEP.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_MENS_DEP.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_MENS_DEP.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_MENS_DEP.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_EMP_TIT
                    ON EVH_EMP_TIT.NREVENTOM = BNH.NREVETITULAREMP
                    AND
                    (
                        EVH_EMP_TIT.NRORG = BNM.NRORG
                        OR EVH_EMP_TIT.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_EMP_TIT.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_EMP_TIT.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_EMP_TIT.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_EMP_DEP
                    ON EVH_EMP_DEP.NREVENTOM = BNH.NREVEDEPEMPRESA
                    AND
                    (
                        EVH_EMP_DEP.NRORG = BNM.NRORG
                        OR EVH_EMP_DEP.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_EMP_DEP.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_EMP_DEP.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_EMP_DEP.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_TIT_VINC
                    ON EVH_TIT_VINC.NREVENTOM = BNH.NREVETITULARVINC
                    AND
                    (
                        EVH_TIT_VINC.NRORG = BNM.NRORG
                        OR EVH_TIT_VINC.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_TIT_VINC.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_TIT_VINC.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_TIT_VINC.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_DEP_VINC
                    ON EVH_DEP_VINC.NREVENTOM = BNH.NREVEDEPEVINC
                    AND
                    (
                        EVH_DEP_VINC.NRORG = BNM.NRORG
                        OR EVH_DEP_VINC.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_DEP_VINC.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_DEP_VINC.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_DEP_VINC.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_TIT_COPAR
                    ON EVH_TIT_COPAR.NREVENTOM = BNH.NREVENTOCOPART
                    AND
                    (
                        EVH_TIT_COPAR.NRORG = BNM.NRORG
                        OR EVH_TIT_COPAR.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_TIT_COPAR.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_TIT_COPAR.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_TIT_COPAR.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_DEP_COPAR
                    ON EVH_DEP_COPAR.NREVENTOM = BNH.NREVENTOCOPARTDEP
                    AND
                    (
                        EVH_DEP_COPAR.NRORG = BNM.NRORG
                        OR EVH_DEP_COPAR.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_DEP_COPAR.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_DEP_COPAR.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_DEP_COPAR.NREVENTOM
                    )
                LEFT JOIN
                    FPA_EVENTOH EVH_TAXA_INCL_IMPL
                    ON EVH_TAXA_INCL_IMPL.NREVENTOM = BNH.NREVENTOTAXA
                    AND
                    (
                        EVH_TAXA_INCL_IMPL.NRORG = BNM.NRORG
                        OR EVH_TAXA_INCL_IMPL.NRORG = :NRORG_PADRAO
                    )
                    AND EVH_TAXA_INCL_IMPL.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(SUB_EVENTH.DTMESCOMPETENC)
                        FROM
                            FPA_EVENTOH SUB_EVENTH
                        WHERE
                            SUB_EVENTH.NRORG = EVH_TAXA_INCL_IMPL.NRORG
                            AND SUB_EVENTH.DTMESCOMPETENC <= :DTMESCOMPETENC
                            AND SUB_EVENTH.NREVENTOM = EVH_TAXA_INCL_IMPL.NREVENTOM
                    )
            WHERE
                BNM.NRORG = :NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $beneficios = $std->fetchAll();

        return $beneficios;
    }

    public function getTipoVinculo(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                TVH.*,
                TVM.DTINIVIGENCIA,
                TVM.DTFIMVIGENCIA
            FROM
                GPE_TIPOVINCULOM TVM
                INNER JOIN
                GPE_TIPOVINCULOH TVH ON
                    TVH.NRTIPOVINCULOM = TVM.NRTIPOVINCULOM AND
                    (
                        TVM.NRORG = TVH.NRORG OR
                        TVH.NRORG = :P_NRORG_PADRAO
                    ) AND
                    TVH.DTMESCOMPETENC = (
                        SELECT
                            MAX(DTMESCOMPETENC)
                        FROM
                            GPE_TIPOVINCULOH H1
                        WHERE
                            H1.NRTIPOVINCULOM = TVH.NRTIPOVINCULOM AND
                            H1.NRORG = TVH.NRORG AND
                            H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                        )
            WHERE
                TVM.NRORG = :P_NRORG OR
                TVM.NRORG = :P_NRORG_PADRAO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }

    public function getTipoAdmissao(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                FPA_TPADMISSAO
            WHERE
                NRORG = :P_NRORG OR
                NRORG = :P_NRORG_PADRAO
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }

    public function getVinculoEmpregaticio(): array {
        $sql = <<<SQL
            SELECT
                VE.*,
                CT.DSCATEGESOC
            FROM
                GPE_VINCULOEMPREG VE
                LEFT JOIN
                GPE_CATEGTRABESOC CT ON
                    VE.NRCATEGESOC = CT.NRCATEGESOC
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute();

        $vinculosEmp = $std->fetchAll();

        return $vinculosEmp;
    }

    public function getTipoOcupacao(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
                *
            FROM
                GPE_TIPOOCUPACAO
            WHERE
                NRORG IN (:P_NRORG, :P_NRORG_PADRAO)
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }

    public function getOcupacao($params): array {
        $sql = <<<SQL
            SELECT
                OH.*,
                OM.DTINIVIGENCIA,
                OM.DTFIMVIGENCIA,
                OM.NRTIPOOCUPACAO,
                C.NMCBO,
                TPO.NMTIPOOCUPACAO
            FROM
                GPE_OCUPACAOM OM
                INNER JOIN
                GPE_OCUPACAOH OH ON
                    OM.NROCUPACAOM = OH.NROCUPACAOM AND
                    OH.NRORG = OM.NRORG AND
                    OH.DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(OHH.DTMESCOMPETENC)
                        FROM
                            GPE_OCUPACAOH OHH
                        WHERE
                            OHH.NRORG = :P_NRORG AND
                            OHH.NROCUPACAOM = OH.NROCUPACAOM AND
                            OHH.DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                    )
                INNER JOIN
                GPE_TIPOOCUPACAO TPO ON
                    OM.NRTIPOOCUPACAO = TPO.NRTIPOOCUPACAO AND
                    (
                        TPO.NRORG = OM.NRORG OR
                        TPO.NRORG = :P_NRORG_PADRAO
                    )
                LEFT JOIN
                GPE_CBO C ON
                    C.NRCBO = OH.NRCBO AND
                    (
                        C.NRORG = OH.NRORG OR
                        C.NRORG = :P_NRORG_PADRAO OR
                        C.NRORG IS NULL
                    )
            WHERE
                OM.NRORG = :P_NRORG AND
                (
                    :P_VINGENCIA = 'V' AND
                    (
                        DTFIMVIGENCIA >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR
                        DTFIMVIGENCIA IS NULL
                    ) OR
                    :P_VINGENCIA = 'T' AND
                    (
                        DTFIMVIGENCIA IS NOT NULL AND
                        DTFIMVIGENCIA < TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                    ) OR
                    (
                        :P_VINGENCIA <> 'T' AND
                        :P_VINGENCIA <> 'V'
                    )
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }

    public function getEstruturaGerLeg(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                H.NRESTRUTURAH,
                H.CDCAEPFESTRUT,
                H.CDCNPJESTRUT,
                H.NRESTRUTURAM,
                H.NMESTRUTURAH,
                M.NRTIPOESTRUTURA,
                H.NRTPOBRIGAPRENDIZ,
                H.NRTPOBRIGAPCD,
                H.NRORG,
                H.NRPARCREGAMB,
                H.NRPARCMONBIO,
                H.DTINIRESAMB,
                H.NMRAZSOCESTRUT,
                M.NMESTRUTURAM,
                H.DTMESCOMPETENC,
                H.DTINCLUSAO,
                H.CDOPERINCLUSAO,
                H.NRORGINCLUSAO,
                H.DTULTATU,
                H.CDOPERULTATU,
                H.NRORGULTATU,
                M.IDATIVO,
                H.DTFIMRESAMB,
                T.NMTIPOESTRUTURA,
                EP.IDUTILIZAMATRIC,
                ESTRUT_LEGAL.NRESTRUTLEGAL,
                ESTRUT_LEGAL.NMRAZSOCESTRUT_LEGAL,
                ESTRUT_LEGAL.NRTIPOESTRUTURA_LEGAL,
                ESTRUT_LEGAL.IDUTILIZAMATRIC_LEGAL
            FROM
                ESTRUTURAM M
                LEFT JOIN
                    ESTRUTURAH H
                    ON H.NRESTRUTURAM = M.NRESTRUTURAM
                    AND H.NRORG = M.NRORG
                    AND
                    (
                        M.NRTIPOESTRUTURA = :NRTIPOESTRUTURA
                        OR :NRTIPOESTRUTURA IS NULL
                    )
                JOIN
                    TIPOESTRUTURA T
                    ON M.NRTIPOESTRUTURA = T.NRTIPOESTRUTURA
                LEFT JOIN
                    ESO_PARAMETRO EP
                    ON EP.NRESTRUTURAM = M.NRESTRUTURAM
                    AND EP.NRORG = M.NRORG
                LEFT JOIN
                    (
                        SELECT DISTINCT
                            TPH.NRTIPOESTRUTSUP,
                            TPH.NRTIPOESTRUTREL,
                            TPH.NRORG,
                            HO.NRESTRUTURAMREL,
                            ESTRUT_SUP.NRESTRUTURAM AS NRESTRUTLEGAL,
                            ESTRUT_SUP.NMESTRUTURAM AS NMRAZSOCESTRUT_LEGAL,
                            ESTRUT_SUP.NRTIPOESTRUTURA AS NRTIPOESTRUTURA_LEGAL,
                            EP_SUP.IDUTILIZAMATRIC AS IDUTILIZAMATRIC_LEGAL
                        FROM
                            TIPOHIERARQORG TPH
                            JOIN
                                HIERARQUIAORG HO
                                ON HO.NRTIPOHIERARQORG = TPH.NRTIPOHIERARQORG
                                AND
                                (
                                    HO.NRORG = :NRORG
                                    OR HO.NRORG = :NRORG_PADRAO
                                )
                            JOIN
                                ESTRUTURAM ESTRUT_SUP
                                ON ESTRUT_SUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                                AND ESTRUT_SUP.NRORG = TPH.NRORG
                            LEFT JOIN
                                ESO_PARAMETRO EP_SUP
                                ON EP_SUP.NRESTRUTURAM = ESTRUT_SUP.NRESTRUTURAM
                                AND EP_SUP.NRORG = ESTRUT_SUP.NRORG
                        WHERE
                            TPH.NRORG = :NRORG
                    )
                    ESTRUT_LEGAL
                    ON ESTRUT_LEGAL.NRTIPOESTRUTREL = T.NRTIPOESTRUTURA
                    AND ESTRUT_LEGAL.NRESTRUTURAMREL = M.NRESTRUTURAM
                    AND ESTRUT_LEGAL.NRTIPOESTRUTURA_LEGAL = :NRTPESTRUTLEGAL
            WHERE
                H.NRORG = :NRORG
                AND
                (
                    M.DTFIMVIGENCIA >= :DTMESCOMPETENC
                    OR M.DTFIMVIGENCIA IS NULL
                )
                AND H.DTMESCOMPETENC =
                (
                    SELECT
                        MAX(H1.DTMESCOMPETENC)
                    FROM
                        ESTRUTURAH H1
                    WHERE
                        H1.NRORG = H.NRORG
                        AND H1.NRESTRUTURAM = H.NRESTRUTURAM
                        AND H1.DTMESCOMPETENC <= :DTMESCOMPETENC
                )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $estruturas = $std->fetchAll();

        return $estruturas;
    }

    public function getEstruturaLegal(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                EH.*,
                EM.NRTPEVENTOCTBL,
                EM.DTESOCIAL,
                EM.NRREGIMEAPUR,
                EM.DTINIVIGENCIA,
                EM.DTFIMVIGENCIA,
                EM.NRTIPOESTRUTURA,
                EM.CDINTESTRUTURA,
                EM.NMESTRUTURAM,
                EM.CDINTESTRUTEXPFIL,
                EM.CDINTESTRUTEXP,
                EM.NRGRUPOESOCIAL,
                PN.CDTIPOINSCRICAO,
                TIPOESTRUTURA.NMTIPOESTRUTURA,
                CT.DSCLASSTRIBUT,
                CA.DSCALENDARIO,
                CN.DSCNAE,
                CN.VRALIQRAT,
                FPAS.DSFPAS,
                PP.NMPRINCIPALPARC AS NMPROPRIEMPR,
                PRI.NMPRINCIPALPARC AS NMPARCRESPINF,
                PRL.NMPRINCIPALPARC AS NMPARCENTISIN,
                PRA.NMPRINCIPALPARC AS NMPARCREGAMB,
                PMB.NMPRINCIPALPARC AS NMPARCMONBIO,
                NJ.DSNATUJURI,
                ESO_PARAMETRO.IDUTILIZAMATRIC
            FROM
                ESTRUTURAM EM
                JOIN
                    ESTRUTURAH EH
                    ON EM.NRESTRUTURAM = EH.NRESTRUTURAM
                    AND EM.NRORG = EH.NRORG
                    AND DTMESCOMPETENC =
                    (
                        SELECT
                            MAX(EHH.DTMESCOMPETENC)
                        FROM
                            ESTRUTURAH EHH
                        WHERE
                            EHH.NRORG = :NRORG
                            AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                            AND EHH.DTMESCOMPETENC <= TO_DATE(:DTMESCOMPETENC, 'DD/MM/YYYY')
                    )
                JOIN
                    PARCNEGOCIO PN
                    ON PN.NRORG = EM.NRORG
                    AND PN.NRPARCNEGOCIO = EM.NRPARCNEGOCIO
                JOIN
                    TIPOESTRUTURA
                    ON EM.NRTIPOESTRUTURA = TIPOESTRUTURA.NRTIPOESTRUTURA
                LEFT JOIN
                    CLASSTRIBUT CT
                    ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT
                    AND
                    (
                        CT.NRORG = EM.NRORG
                        OR CT.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    CALENDARIO CA
                    ON CA.NRCALENDARIO = EH.NRCALENDARIO
                    AND
                    (
                        CA.NRORG = EM.NRORG
                        OR CA.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    CNAEFISCAL CN
                    ON CN.CDCNAE = EH.CDCNAE
                    AND
                    (
                        CN.NRORG = EM.NRORG
                        OR CN.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    FPAS
                    ON FPAS.CDFPAS = EH.CDFPAS
                    AND
                    (
                        FPAS.NRORG = EM.NRORG
                        OR FPAS.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    PARCNEGOCIO PP
                    ON EH.NRPROPRIEMPR = PP.NRPARCNEGOCIO
                    AND EM.NRORG = PP.NRORG
                LEFT JOIN
                    PARCNEGOCIO PRI
                    ON EH.NRPARCRESPINF = PRI.NRPARCNEGOCIO
                    AND EM.NRORG = PRI.NRORG
                LEFT JOIN
                    PARCNEGOCIO PRL
                    ON EH.NRPARCENTISIN = PRL.NRPARCNEGOCIO
                    AND EM.NRORG = PRL.NRORG
                LEFT JOIN
                    PARCNEGOCIO PRA
                    ON EH.NRPARCREGAMB = PRA.NRPARCNEGOCIO
                    AND EM.NRORG = PRA.NRORG
                LEFT JOIN
                    PARCNEGOCIO PMB
                    ON EH.NRPARCMONBIO = PMB.NRPARCNEGOCIO
                    AND EM.NRORG = PMB.NRORG
                LEFT JOIN
                    NATUREZAJURI NJ
                    ON EH.CDNATUJURI = NJ.CDNATUJURI
                    AND
                    (
                        EH.NRORG = NJ.NRORG
                        OR NJ.NRORG = :NRORG_PADRAO
                    )
                LEFT JOIN
                    ESO_PARAMETRO
                    ON ESO_PARAMETRO.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND ESO_PARAMETRO.NRORG = EM.NRORG
                JOIN
                    GPE_PARAMORG PARAM
                    ON PARAM.NRORG = EM.NRORG
                    AND PARAM.NRTPESTRUTLEGAL = EM.NRTIPOESTRUTURA
            WHERE
                EM.NRORG = :NRORG
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $tipos = $std->fetchAll();

        return $tipos;
    }
}