<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Util\NovoCodigo;
use Ems\Repository\PeriodoRepository;
use Ems\Util\Utilities;

class OcorrenciaRepository {
    public function __construct(EntityManager $entityManager, PeriodoRepository $periodoRepository) {
        $this->connection           = $entityManager->getConnection();
        $this->periodoRepository    = $periodoRepository;
        $this->entityManager        = $entityManager;
    }

    public function getOcorrencia(
        int $nrOrg,
        int $nrOcorrencia
    ): array {
        $params = [
            'P_NRORG'        => $nrOrg,
            'P_NROCORRENCIA' => $nrOcorrencia
        ];

        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRORG = :P_NRORG
              AND NRPROGOCORRENCIA = :P_NROCORRENCIA
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $ocorrencia = $std->fetch();

        return $ocorrencia;
    }

    public function getTipoProgocorrencia(string $nrorg, string $nrtipoprogocorrencia) : string {
        try {
            $sql = <<<SQL
                SELECT *
                FROM PTO_TIPOPROGOCORRENCIA TP
                WHERE (TP.NRORG = :P_NRORG OR TP.NRORG = :P_NRORG_PADRAO)
                            AND TP.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA
            ORDER BY TP.NMTIPOPROGOCORRENCIA
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRTIPOPROGOCORRENCIA"    => $nrtipoprogocorrencia,
                    "P_NRORG_PADRAO"            => 0
                )
            );

            $result = $query->fetchAll();

            $idprogocorconfirmada = '';
            if ($result[0]['IDPROGOCORACONFIRMAR'] == "S") {
                $idprogocorconfirmada = "P";
            } else {
                $idprogocorconfirmada = "S";
            }
            return $idprogocorconfirmada;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Rotina que salva as ocorrências de marcação de ponto
     * 
     * 
     *  Campos do tipo string são NOT_NULL
     */
    public function geraOcorrenciaFrequencia(
        string  $nrvinculom, 
        string  $dtinicioprogocor, 
                $dtfimprogocor, 
                $iddiassemana, 
                $idtipocalendario, 
        string  $idtipohorario, 
                $nrhordiam, 
                $nrintervalohorario, 
                $nrmotivoocorfreq, 
                $qtmaximaprogocor, 
                $dsobservacao, 
                $nrtipoprogocorrencia, 
                $nrmarcacaoponto, 
        string  $idprogocorconfirmada, 
                $dsobservacaorescusa, 
                $hrprojetarintervalonprog, 
                $dtinclusao, 
                $cdoperinclusao, 
        string  $nrorg
    ) : int {
        
        try {
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, false);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDDIASSEMANA,
                    IDTIPODIACALENDARIO,
                    IDTIPOHORARIO,
                    NRHORDIAM,
                    NRINTERVALOHORARIO,
                    NRMOTIVOOCORFREQ,
                    QTMAXIMAPROGOCOR,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    NRMARCACAOPONTO,
                    IDPROGOCORCONFIRMADA,
                    DSOBSERVACAORECUSA,
                    HRPROJETARINTERVALONPROG,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORG
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDDIASSEMANA,
                    :P_IDTIPOCALENDARIO,
                    :P_IDTIPOHORARIO,
                    :P_NRHORDIAM,
                    :P_NRINTERVALOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_QTMAXIMAPROGOCOR,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_NRMARCACAOPONTO,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_DSOBSERVACAORESCUSA,
                    :P_HRPROJETARINTERVALONPROG,
                    TO_DATE(:P_DTINCLUSAO, 'DD/MM/YYYY'),
                    :P_CDOPERINCLUSAO,
                    :P_NRORG
                )
SQL;

            $query = $this->connection->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                    "P_NRVINCULOM"                  => $nrvinculom,
                    "P_DTINICIOPROGOCOR"            => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"               => $dtfimprogocor,
                    "P_IDDIASSEMANA"                => $iddiassemana,
                    "P_IDTIPOCALENDARIO"            => $idtipocalendario,
                    "P_IDTIPOHORARIO"               => $idtipohorario,
                    "P_NRHORDIAM"                   => $nrhordiam,
                    "P_NRINTERVALOHORARIO"          => $nrintervalohorario,
                    "P_NRMOTIVOOCORFREQ"            => $nrmotivoocorfreq,
                    "P_QTMAXIMAPROGOCOR"            => $qtmaximaprogocor,
                    "P_DSOBSERVACAO"                => $dsobservacao,
                    "P_NRTIPOPROGOCORRENCIA"        => $nrtipoprogocorrencia,
                    "P_NRMARCACAOPONTO"             => $nrmarcacaoponto,
                    "P_IDPROGOCORCONFIRMADA"        => $idprogocorconfirmada,
                    "P_DSOBSERVACAORESCUSA"         => $dsobservacaorescusa,
                    "P_HRPROJETARINTERVALONPROG"    => $hrprojetarintervalonprog,
                    "P_DTINCLUSAO"                  => $dtinclusao,
                    "P_CDOPERINCLUSAO"              => $cdoperinclusao,
                    "P_NRORG"                       => $nrorg
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFrequencia(string $nrvinculom, string $dtinicioprogocor, string $dtfimprogocor, string $nrtipoprogocorrencia, string $nrmarcacaoponto, string $nrprogocorrencia, string $nrorg) : array {
        try {
            if ($nrmarcacaoponto != '') {
                $condition = "AND NRMARCACAOPONTO = $nrmarcacaoponto";
            } elseif ($nrprogocorrencia != '') {
                $condition = "AND NRPROGOCORRENCIA = $nrprogocorrencia";
            } elseif ($nrtipoprogocorrencia != '') {
                $condition = "AND NRTIPOPROGOCORRENCIA = $nrtipoprogocorrencia";
            } else {
                $condition = "AND 1 = 1";
            }

            $sql = <<<SQL
                DELETE FROM
                    PTO_PROGOCORRENCIA
                WHERE NRORG                = $nrorg
                    AND NRVINCULOM           = $nrvinculom
                    AND DTINICIOPROGOCOR     = TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY')
                    AND DTFIMPROGOCOR        = TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY')
                    $condition
SQL;
            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"    => $dtinicioprogocor,
                    "P_DTFIMPROGOCOR"       => $dtfimprogocor
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasPeriodo(string $nrorg, string $nrvinculom, string $nrperiodoapuracao, string $idprogocorconfirmada = null) {
        try {
            if (empty($nrperiodoapuracao)) {
                $periodo = $this->periodoRepository->getPeriodoApuracaoAtual($nrvinculom, $nrorg);
            } else {
                $periodo = $this->periodoRepository->getPeriodo($nrorg, $nrperiodoapuracao);
            }

            if (empty($periodo)) {
                throw new \Exception('Período não encontrado.');
            }

            $dtinicialapuracao = Utilities::formatDate($periodo['DTINICIALAPURACAO']);

            $dtfinalapuracao = Utilities::formatDate($periodo['DTFINALAPURACAO']);

            $sql = <<<SQL
                SELECT
                    PO.NRORG,
                    PO.NRPROGOCORRENCIA,
                    PO.NRVINCULOM,
                    VM.NMVINCULOM,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    TO_CHAR(PO.DTFIMPROGOCOR, 'DD/MM/YYYY') AS DTFIMPROGOCOR,
                    ((PO.DTFIMPROGOCOR - PO.DTINICIOPROGOCOR)  +1) AS DIAOCORRE,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRMOTIVOOCORFREQ,
                    PO.NRHORDIAM,
                    PO.NRSITUFREQUENCIA,
                    PO.NRSITUFREQGATILHO,
                    PO.QTMINGATILHOPROGOCOR,
                    PO.QTMAXGATILHOPROGOCOR,
                    PO.VRCOEFICIENTEACRESCIMO,
                    PO.IDPROGOCORRENCIA,
                    PO.QTMINIMAPROGOCOR,
                    PO.QTMAXIMAPROGOCOR,
                    PO.DSOBSERVACAO,
                    PO.IDATIVO,
                    PO.NRORGINCLUSAO,
                    PO.DTINCLUSAO,
                    PO.CDOPERINCLUSAO,
                    PO.NRORGULTATU,
                    PO.DTULTATU,
                    PO.CDOPERULTATU,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMARCACAOPONTO,
                    PO.NRINTERVALOHORARIO,
                    PO.IDPROGOCORCONFIRMADA,
                    PO.CDOPERVERIFICADO,
                    PO.DSOBSERVACAORECUSA,
                    PO.HRPROJETARINTERVALONPROG,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR,
                    PO.DSPATH,
                    TO_CHAR(MP.DTHRMARCACAOPONTO, 'HH24:MI') HRINICIO,
                    DECODE (PO.IDPROGOCORCONFIRMADA, 'S', 'Confirmada',
                                     'N', 'Rejeitada',
                                     'P', 'Pendente',
                                     'Pendente') DSCONFIRM
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                LEFT JOIN GPE_VINCULOM VM ON VM.NRORG = PO.NRORG AND VM.NRVINCULOM = PO.NRVINCULOM
                LEFT JOIN PTO_TIPOPROGOCORRENCIA TP ON (TP.NRORG = PO.NRORG OR TP.NRORG = 0) AND TP.NRTIPOPROGOCORRENCIA = PO.NRTIPOPROGOCORRENCIA
                LEFT JOIN GPE_MARCACAOPONTO MP ON MP.NRORG = PO.NRORG AND MP.NRMARCACAOPONTO = PO.NRMARCACAOPONTO
                WHERE PO.NRORG = :P_NRORG
                    AND (PO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND (PO.IDPROGOCORCONFIRMADA = :P_IDPROGOCORCONFIRMADA OR :P_IDPROGOCORCONFIRMADA IS NULL)
                    AND PO.DTINICIOPROGOCOR BETWEEN TO_DATE(:P_DTINICIALAPURACAO, 'DD/MM/YYYY') AND TO_DATE(:P_DTFINALAPURACAO, 'DD/MM/YYYY')
                    AND (TP.NRTIPOPROGOCORRENCIA IN (1,4,5,10,13,16,17,22,23,24,25) OR TP.NRTIPOPROGOCORRENCIA IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrvinculom,
                    "P_DTINICIALAPURACAO"       => $dtinicialapuracao,
                    "P_DTFINALAPURACAO"         => $dtfinalapuracao,
                    "P_IDPROGOCORCONFIRMADA"    => $idprogocorconfirmada
                )
            );

            $resultQuery = $query->fetchAll();


            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Função que retorna as ocorrências encontradas para um array de vínculos em um período específico. Usa a função '$this->getOcorrenciasPeriodo' para buscar as ocorrências. Recebe um array com os numeros de vinculo
     *
     *  @param string $nrorg
     *  @param Array  $arrayNrvinculom
     *  @param string $nrperiodoapuracao
     *  @param string $idprogocorconfirmada
     */
    public function getOcorrenciasPeriodoPorVinculos(string $nrorg, Array $arrayNrvinculom, string $nrperiodoapuracao, string $idprogocorconfirmada = null) {
        $ocorrenciasVinculos = [];
        foreach($arrayNrvinculom as $nrvinculom) {
            $ocorrenciasVinculo = $this->getOcorrenciasPeriodo($nrorg, $nrvinculom, $nrperiodoapuracao, $idprogocorconfirmada);
            if (count($ocorrenciasVinculo) > 0) {
                $ocorrenciasVinculos = array_merge($ocorrenciasVinculos, $ocorrenciasVinculo);
            }
        }

        return $ocorrenciasVinculos;
    }


    public function getMotivoOcorFreq(string $nrorg) {
        try {
            $sql = <<<SQL
                SELECT DSMOTIVOOCORFREQ,
                       NRMOTIVOOCORFREQ
                FROM PTO_MOTIVOOCORFREQ
                WHERE NRORG = :P_NRORG
                ORDER BY DSMOTIVOOCORFREQ
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTpOcorrencia($nrorg)  {
        try {
            $sql = <<<SQL
                SELECT
                    TP.NRTIPOPROGOCORRENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR
                FROM PTO_TIPOPROGOCORRENCIA TP
                WHERE (TP.NRORG = :P_NRORG OR TP.NRORG = 0)
                AND TP.NRTIPOPROGOCORRENCIA IN (1,4,5,10,13,16,17,22,23,24,25)
            ORDER BY TP.NMTIPOPROGOCORRENCIA
SQL;
            $query = $this->connection->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG" => $nrorg
                )
            );

            $result = $query->fetchAll();

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function newOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];
            $nrprogocorrencia = NovoCodigo::geraCodigo($nrorg, 'PTO_PROGOCORRENCIA', 1, 22, true);

            $sql = <<<SQL
                INSERT INTO PTO_PROGOCORRENCIA (
                    NRPROGOCORRENCIA,
                    NRVINCULOM,
                    DTINICIOPROGOCOR,
                    DTFIMPROGOCOR,
                    IDTIPOHORARIO,
                    NRMOTIVOOCORFREQ,
                    DSOBSERVACAO,
                    NRTIPOPROGOCORRENCIA,
                    IDPROGOCORCONFIRMADA,
                    NRMARCACAOPONTO,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORG,
                    DSPATH,
                    IDDIASSEMANA,
                    IDTIPODIACALENDARIO
                ) VALUES (
                    :P_NRPROGOCORRENCIA,
                    :P_NRVINCULOM,
                    TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                    TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                    :P_IDTIPOHORARIO,
                    :P_NRMOTIVOOCORFREQ,
                    :P_DSOBSERVACAO,
                    :P_NRTIPOPROGOCORRENCIA,
                    :P_IDPROGOCORCONFIRMADA,
                    :P_NRMARCACAOPONTO,
                    TO_DATE(:P_DTINCLUSAO, 'DD/MM/YYYY'),
                    :P_CDOPERINCLUSAO,
                    :P_NRORG,
                    :P_DSPATH,
                    :P_IDDIASSEMANA,
                    :P_IDTIPODIACALENDARIO
                )
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA"            => $nrprogocorrencia,
                    "P_NRVINCULOM"                  => isset($row['NRVINCULOM'])            ? $row['NRVINCULOM'] : NULL ,
                    "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                    "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                    "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                    "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                    "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                    "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                    "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : 'P' ,
                    "P_NRMARCACAOPONTO"             => isset($row['NRMARCACAOPONTO'])       ? $row['NRMARCACAOPONTO'] : NULL ,
                    "P_DTINCLUSAO"                  => isset($row['DTINCLUSAO'])            ? $row['DTINCLUSAO'] : date("d/m/Y") ,
                    "P_CDOPERINCLUSAO"              => isset($row['CDOPERINCLUSAO'])        ? $row['CDOPERINCLUSAO'] : 1,
                    "P_DSPATH"                      => isset($row['DSPATH'])                ? $row['DSPATH'] : NULL,
                    "P_NRORG"                       => $nrorg,
                    "P_IDDIASSEMANA"                => isset($row['IDDIASSEMANA'])          ? $row['IDDIASSEMANA'] : NULL,
                    "P_IDTIPODIACALENDARIO"         => isset($row['IDTIPODIACALENDARIO'])          ? $row['IDTIPODIACALENDARIO'] : NULL,
                )
            );

            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                UPDATE PTO_PROGOCORRENCIA
                    SET
                        DTINICIOPROGOCOR      =  TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY'),
                        DTFIMPROGOCOR         =  TO_DATE(:P_DTFIMPROGOCOR, 'DD/MM/YYYY'),
                        IDTIPOHORARIO         =  :P_IDTIPOHORARIO,
                        NRMOTIVOOCORFREQ      =  :P_NRMOTIVOOCORFREQ,
                        DSOBSERVACAO          =  :P_DSOBSERVACAO,
                        NRTIPOPROGOCORRENCIA  =  :P_NRTIPOPROGOCORRENCIA,
                        IDPROGOCORCONFIRMADA  =  :P_IDPROGOCORCONFIRMADA
                    WHERE
                        NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_DTINICIOPROGOCOR"            => isset($row['DTINICIOPROGOCOR'])      ? $row['DTINICIOPROGOCOR'] : NULL ,
                    "P_DTFIMPROGOCOR"               => isset($row['DTFIMPROGOCOR'])         ? $row['DTFIMPROGOCOR'] : NULL ,
                    "P_IDTIPOHORARIO"               => isset($row['IDTIPOHORARIO'])         ? $row['IDTIPOHORARIO'] : 'N' ,
                    "P_NRMOTIVOOCORFREQ"            => isset($row['NRMOTIVOOCORFREQ'])      ? $row['NRMOTIVOOCORFREQ'] : NULL ,
                    "P_DSOBSERVACAO"                => isset($row['DSOBSERVACAO'])          ? $row['DSOBSERVACAO'] : NULL ,
                    "P_NRTIPOPROGOCORRENCIA"        => isset($row['NRTIPOPROGOCORRENCIA'])  ? $row['NRTIPOPROGOCORRENCIA'] : NULL ,
                    "P_IDPROGOCORCONFIRMADA"        => isset($row['IDPROGOCORCONFIRMADA'])  ? $row['IDPROGOCORCONFIRMADA'] : NULL ,
                    "P_NRPROGOCORRENCIA"            => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_NRORG"                       => $nrorg
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function removeOcorrenciaFreq($row) {
        try {
            $nrorg = $row['NRORG'];

            $sql = <<<SQL
                DELETE FROM PTO_PROGOCORRENCIA
                    WHERE
                    NRPROGOCORRENCIA = :P_NRPROGOCORRENCIA AND NRORG = :P_NRORG
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $result = $query->execute(
                array(
                    "P_NRPROGOCORRENCIA" => isset($row['NRPROGOCORRENCIA'])      ? $row['NRPROGOCORRENCIA'] : NULL ,
                    "P_NRORG"            => $nrorg
                )
            );
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasPorTipoOcorrencia(
        string  $nrvinculom,
        string  $nrorg,
        string  $nrtipoprogocorrencia,
        string  $dtinicioprogocor   = null,
        string  $nrmarcacaoponto    = null
    ): array {
        try {
            $sql = <<<SQL
                SELECT
                    *
                FROM
                    PTO_PROGOCORRENCIA OCOR
                WHERE
                    OCOR.NRORG                      = :P_NRORG
                    AND OCOR.NRTIPOPROGOCORRENCIA   = :P_NRTIPOPROGOCORRENCIA
                    AND OCOR.NRVINCULOM             = :P_NRVINCULOM
                    AND (TO_CHAR(OCOR.DTINICIOPROGOCOR, 'DD/MM/YYYY')   = :P_DTINICIOPROGOCOR   OR :P_DTINICIOPROGOCOR  IS NULL)
                    AND (OCOR.NRMARCACAOPONTO                           = :P_NRMARCACAOPONTO    OR :P_NRMARCACAOPONTO   IS NULL)
SQL;

            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    'P_NRORG'                   => $nrorg,
                    'P_NRTIPOPROGOCORRENCIA'    => $nrtipoprogocorrencia,
                    'P_NRVINCULOM'              => $nrvinculom,
                    'P_DTINICIOPROGOCOR'        => $dtinicioprogocor,
                    'P_NRMARCACAOPONTO'         => $nrmarcacaoponto
                )
            );

            return $query->fetchAll();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getOcorrenciasDia(
        string  $nrorg,
        string  $nrvinculom,
                $dtinicioprogocor,
        string  $nrtipoprogocorrencia = null
    ){
        
        try {

            $sql = <<<SQL
                SELECT
                    PO.NRORG,
                    PO.NRPROGOCORRENCIA,
                    PO.NRVINCULOM,
                    TO_CHAR(PO.DTINICIOPROGOCOR, 'DD/MM/YYYY') AS DTINICIOPROGOCOR,
                    TO_CHAR(PO.DTFIMPROGOCOR, 'DD/MM/YYYY') AS DTFIMPROGOCOR,
                    ((PO.DTFIMPROGOCOR - PO.DTINICIOPROGOCOR)  +1) AS DIAOCORRE,
                    PO.IDDIASSEMANA,
                    PO.IDTIPODIACALENDARIO,
                    PO.IDTIPOHORARIO,
                    PO.NRMOTIVOOCORFREQ,
                    PO.NRHORDIAM,
                    PO.NRSITUFREQUENCIA,
                    PO.NRSITUFREQGATILHO,
                    PO.QTMINGATILHOPROGOCOR,
                    PO.QTMAXGATILHOPROGOCOR,
                    PO.VRCOEFICIENTEACRESCIMO,
                    PO.IDPROGOCORRENCIA,
                    PO.QTMINIMAPROGOCOR,
                    PO.QTMAXIMAPROGOCOR,
                    PO.DSOBSERVACAO,
                    PO.IDATIVO,
                    PO.NRORGINCLUSAO,
                    PO.DTINCLUSAO,
                    PO.CDOPERINCLUSAO,
                    PO.NRORGULTATU,
                    PO.DTULTATU,
                    PO.CDOPERULTATU,
                    PO.NRTIPOPROGOCORRENCIA,
                    PO.NRMARCACAOPONTO,
                    PO.NRINTERVALOHORARIO,
                    PO.IDPROGOCORCONFIRMADA,
                    PO.CDOPERVERIFICADO,
                    PO.DSOBSERVACAORECUSA,
                    PO.HRPROJETARINTERVALONPROG,
                    MO.DSMOTIVOOCORFREQ,
                    SF.DSSITUFREQUENCIA,
                    TP.NMTIPOPROGOCORRENCIA,
                    TP.IDPROGOCORACONFIRMAR,
                    PO.DSPATH,
                    DECODE (PO.IDPROGOCORCONFIRMADA, 'S', 'Confirmada',
                                     'N', 'Rejeitada',
                                     'P', 'Pendente',
                                     'Pendente') DSCONFIRM
                FROM PTO_PROGOCORRENCIA PO
                LEFT JOIN PTO_MOTIVOOCORFREQ MO ON MO.NRORG = PO.NRORG AND MO.NRMOTIVOOCORFREQ = PO.NRMOTIVOOCORFREQ
                LEFT JOIN PTO_SITUFREQUENCIA SF ON SF.NRORG = PO.NRORG AND SF.NRSITUFREQUENCIA = PO.NRSITUFREQUENCIA
                LEFT JOIN PTO_TIPOPROGOCORRENCIA TP ON (TP.NRORG = PO.NRORG OR TP.NRORG = 0) AND TP.NRTIPOPROGOCORRENCIA = PO.NRTIPOPROGOCORRENCIA
                WHERE PO.NRORG = :P_NRORG
                    AND (PO.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
                    AND PO.DTINICIOPROGOCOR = TO_DATE(:P_DTINICIOPROGOCOR, 'DD/MM/YYYY')
                    AND (PO.NRTIPOPROGOCORRENCIA = :P_NRTIPOPROGOCORRENCIA OR :P_NRTIPOPROGOCORRENCIA IS NULL)
SQL;
            $query = $this->entityManager->getConnection()->prepare($sql);

            $query->execute(
                array(
                    "P_NRORG"                   => $nrorg,
                    "P_NRVINCULOM"              => $nrvinculom,
                    "P_DTINICIOPROGOCOR"        => $dtinicioprogocor,
                    ":P_NRTIPOPROGOCORRENCIA"   => $nrtipoprogocorrencia
                )
            );

            $resultQuery = $query->fetchAll();

            if (isset($resultQuery)) {
                return $resultQuery;
            } else {
                return null;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getAlmoco(array $params) {
        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRTIPOPROGOCORRENCIA IN (20, 26)
              AND NRORG = :P_NRORG
              AND NRVINCULOM = :P_NRVINCULO
              AND DTINICIOPROGOCOR = :P_DATE
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $almoco = $std->fetch();

        return $almoco;
    }
    
    public function getOcorrenciaByNrmarcacao(
        $nrorg,
        $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            SELECT *
            FROM PTO_PROGOCORRENCIA
            WHERE NRORG             = :P_NRORG
              AND NRMARCACAOPONTO   = :P_NRMARCACAOPONTO
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )    
        );

        $ocorrencia = $std->fetch();

        return $ocorrencia;
    }
    
    public function removeOcorrenciaByNrmarcacao(
        $nrorg,
        $nrmarcacaoponto
    ) {
        $sql = <<<SQL
            DELETE FROM
                PTO_PROGOCORRENCIA
            WHERE NRORG                 = :P_NRORG
                AND NRMARCACAOPONTO     = :P_NRMARCACAOPONTO
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $result = $query->execute(
            array(
                'P_NRORG'           => $nrorg,
                'P_NRMARCACAOPONTO' => $nrmarcacaoponto
            )
        );

        return $result;
    }
    
    public function updateOcorrenciaAlmoco(
        $nrorg,
        $nrmarcacaoponto,
        $nrtipoprogocorrencia,
        $hrprojetarintervalonprog    
    ) {
        $sql = <<<SQL
            UPDATE PTO_PROGOCORRENCIA
                SET NRTIPOPROGOCORRENCIA        = :P_NRTIPOPROGOCORRENCIA,
                    HRPROJETARINTERVALONPROG    = :P_HRPROJETARINTERVALONPROG
                WHERE NRMARCACAOPONTO = :P_NRMARCACAOPONTO
                    AND NRORG = :P_NRORG
SQL;
        $query = $this->entityManager->getConnection()->prepare($sql);
        
        $result = $query->execute(
            array(
                'P_NRORG'                       => $nrorg,
                'P_NRMARCACAOPONTO'             => $nrmarcacaoponto,
                'P_NRTIPOPROGOCORRENCIA'        => $nrtipoprogocorrencia,
                'P_HRPROJETARINTERVALONPROG'    => $hrprojetarintervalonprog,
            )
        );

        return $result;
    }
}