<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Mail\SendEmail\SendEmail;
use Util\Repositories;
use Util\Environment;
use Util\DateUtil;
use Util\Messages;
use Util\EmailMessages;

use Ems\Services\VinculoService;
use Ems\Services\ParceiroNegocioService;
use Ems\Services\OperadorService;
use Ems\Services\EstruturaService;
use Ems\Services\ParamOrgService;
use Ems\Services\SolicitacaoService;

class EmailService {
    
    public function __construct(EntityManager $entityManager, Environment $environment, VinculoService $vinculoService, ParceiroNegocioService $parceiroNegocioService, OperadorService $operadorService, EstruturaService $estruturaService, SolicitacaoService $solicitacaoService) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->vinculoService           = $vinculoService;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->operadorService          = $operadorService;
        $this->estruturaService         = $estruturaService;
        $this->solicitacaoService       = $solicitacaoService;
    }
    
    protected const REMETENTE_DEFAULT = 'Teknisa Portal do Gestor';
    protected const TEMPLATE_DEFAULT = 'TEKNISA_TEMPLATE';
    protected const VALIDAEMAILTABELAOPER = true;
    protected const EMPTY_FIELD_DEFAULT_VALUE = 'N/A';
    
    /**
     * Rotina para envio de emails de notificação de ocorrência para os parceiros de negócio associados ao perfil de acesso da estrutura
     * 
     * @param string $cdoperador
     * @param Array  $ocorrencia
     *      array(
     *          IDPROGOCORCONFIRMADA
     *          DTINICIOPROGOCOR
     *          DTFIMPROGOCOR
     *          DSMOTIVOOCORFREQ
     *          DSOBSERVACAO
     *          DATA_ALTERACAOSTATUS
     *          HORA_ALTERACAOSTATUS
     *          NOME_VINCULO
     *      )
     * @param string $baseContent
     * @param string $baseSubject
     * @param string $attachment            Default: null
     * @param bool   $validaEmailTabelaOper Default: self::VALIDAEMAILTABELAOPER
     * @param string $nmremetente           Default: self::REMETENTE_DEFAULT
     * @param string $templateName          Default: self::TEMPLATE_DEFAULT 
     */
    public function sendEmailNotificacaoOcorrencia(
        string $cdoperador,
        Array  $ocorrencia,
        string $baseContent,
        string $baseSubject,
        string $attachment = null,
        bool   $validaEmailTabelaOper       = self::VALIDAEMAILTABELAOPER,
        string $nmremetente                 = self::REMETENTE_DEFAULT,
        string $templateName                = self::TEMPLATE_DEFAULT,
        string $perfilAcessoDestinatarios   = ParamOrgService::PERFIL_ACESS_ADM_DP_RH
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $parceirosNegocio = $this->parceiroNegocioService->getParceirosNegocioPerfilAcesso($perfilAcessoDestinatarios);
        
        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $cdoperador));
        if (!empty($operador)) {
            $parceiroNegocioOperador = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $operador->getNrparcnegocio()));
            $nmoperadornotificacao = $parceiroNegocioOperador->getNmprincipalparc();
        } else {
            throw new \Exception(Messages::MAIL_OPERADOR_NOT_FOUND);
        }
        
        $emailsAEnviar = array();
        
        // Percorre os parceiros montando os emails
        foreach ($parceirosNegocio as $parcnegocio) {
            
            $nrparcnegocio = $parcnegocio['NRPARCNEGOCIO'];
            $email = $this->parceiroNegocioService->getEmailParcNegocio($nrparcnegocio);
            
            if (empty($email)) {
                continue;
            }
            
            $nmdestinatario = $parcnegocio['NMPRINCIPALPARC'];
            $nomevinculoocorrencia = $ocorrencia['NOME_VINCULO'];
            
            $vinculom = $this->vinculoService->getVinculoMestreHistoricoByNrparcnegocio($nrparcnegocio);
            $nrestruturam = $vinculom['NRESTRUTLEGAL'];
            
            
            $contentReplace = array(
                '@NOME_GESTOR@'             => $nmoperadornotificacao,
                '@NOME_VINCULO@'            => $nomevinculoocorrencia,
                '@IDPROGOCORCONFIRMADA@'    => $ocorrencia['IDPROGOCORCONFIRMADA'],
                '@DTINICIOPROGOCOR@'        => $ocorrencia['DTINICIOPROGOCOR'],
                '@DTFIMPROGOCOR@'           => !empty($ocorrencia['DTFIMPROGOCOR'])         ? $ocorrencia['DTFIMPROGOCOR']          : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DSMOTIVOOCORFREQ@'        => !empty($ocorrencia['DSMOTIVOOCORFREQ'])      ? $ocorrencia['DSMOTIVOOCORFREQ']       : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DSOBSERVACAO@'            => !empty($ocorrencia['DSOBSERVACAO'])          ? $ocorrencia['DSOBSERVACAO']           : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DATA_ALTERACAOSTATUS@'    => !empty($ocorrencia['DATA_ALTERACAOSTATUS'])  ? $ocorrencia['DATA_ALTERACAOSTATUS']   : (new \DateTime)->format(DateUtil::FORMATO_DATA),
                '@HORA_ALTERACAOSTATUS@'    => !empty($ocorrencia['HORA_ALTERACAOSTATUS'])  ? $ocorrencia['HORA_ALTERACAOSTATUS']   : (new \DateTime)->format(DateUtil::FORMATO_HORA)
            );
            
            $emailContent = strtr($baseContent, $contentReplace);
            
            
            $subjectReplace = array(
                '@NOME_VINCULO@' => $nomevinculoocorrencia
            );
            
            $emailSubject = strtr($baseSubject, $subjectReplace);
            
            $emailParams = array(
                'templateName' => $templateName,
                'email' => $email,
                'subject' => $emailSubject,
                'nmdestinatario' => $nmdestinatario,
                'nmremetente' => $nmremetente,
                'nrorg' => $nrorg,
                'nrestruturam' => $nrestruturam,
                'content' => $emailContent,
                'attachment' => $attachment,
                'validaEmailTabelaOper' => $validaEmailTabelaOper
            );
            
            $emailsAEnviar[] = $emailParams;
        }
        
        // Envia os emails
        $emailsEnviados = $this->sendEmails($emailsAEnviar);
        
        return $emailsEnviados;
    }
    
    /**
     * Rotina para envio de emails de notificação de programação de férias para os parceiros de negócio associados ao perfil de acesso da estrutura
     * 
     * @param string $cdoperador
     * @param Array  $programacaoFerias
     *      array(
     *          DTINIPREVISAO
     *          QTDIASPREVISAO
     *          DTFINPREVISAO
     *          QTABONOPREVISAO
     *      )
     * @param string $baseContent           Default: EmailMessages::ALTERPROGFERIAS_BODYMESSAGE
     * @param string $baseSubject           Default: EmailMessages::ALTERPROGFERIAS_SUBJECT
     * @param string $attachment            Default: null
     * @param bool   $validaEmailTabelaOper Default: self::VALIDAEMAILTABELAOPER
     * @param string $nmremetente           Default: self::REMETENTE_DEFAULT
     * @param string $templateName          Default: self::TEMPLATE_DEFAULT 
     */
    public function sendEmailNotificacaoProgramacaoFerias(
        string $cdoperador,
        Array  $programacaoFerias,
        string $baseContent                 = EmailMessages::ALTERPROGFERIAS_BODYMESSAGE,
        string $baseSubject                 = EmailMessages::ALTERPROGFERIAS_SUBJECT,
        string $attachment                  = null,
        bool   $validaEmailTabelaOper       = self::VALIDAEMAILTABELAOPER,
        string $nmremetente                 = self::REMETENTE_DEFAULT,
        string $templateName                = self::TEMPLATE_DEFAULT,
        string $perfilAcessoDestinatarios   = ParamOrgService::PERFIL_ACESS_ADM_DP_RH
    ) {
        
        // Dados que serão presentes em todos os emails
        $nrorg = $this->environment->getNrorg();
        
        $parceirosNegocio = $this->parceiroNegocioService->getParceirosNegocioPerfilAcesso($perfilAcessoDestinatarios);
        
        // Operador
        $operador = $this->getDadosOperador($cdoperador, $nrorg);
        
        // Vinculo
        if (empty($programacaoFerias['NRVINCULOM'])) {
            throw new \Exception(Messages::VINC_NOT_FOUND);
        }
        
        $vinculo = $this->getDadosVinculo($programacaoFerias['NRVINCULOM'], $nrorg);
        
        // Unidade
        $unidade = $this->getDadosEstrutura($vinculo['NRESTRUTGEREN'], $nrorg);
        
        $emailsAEnviar = array();
        
        // Percorre os parceiros montando os emails com dados especificos de cada parceiro de negócio
        foreach ($parceirosNegocio as $parcnegocio) {
            
            $nrparcnegocio = $parcnegocio['NRPARCNEGOCIO'];
            $email = $this->parceiroNegocioService->getEmailParcNegocio($nrparcnegocio);
            
            if (empty($email)) {
                continue;
            }
            
            $nmdestinatario = $parcnegocio['NMPRINCIPALPARC'];
            
            $contentReplace = array(
                '@UNIDADE@'         => $unidade['NMESTRUTURAH'],
                '@NOME_GESTOR@'     => $operador['NMOPERADOR'],
                '@EMAIL_GESTOR@'    => $operador['EMAILOPERADOR'],
                '@NOME_VINCULO@'    => $vinculo['NMVINCULOM'],
                '@DTINIPREVISAO@'   => !empty($programacaoFerias['DTINIPREVISAO'])      ? $programacaoFerias['DTINIPREVISAO']   : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@QTDIASPREVISAO@'  => !empty($programacaoFerias['QTDIASPREVISAO'])     ? $programacaoFerias['QTDIASPREVISAO']  : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@DTFINPREVISAO@'   => !empty($programacaoFerias['DTFINPREVISAO'])      ? $programacaoFerias['DTFINPREVISAO']   : self::EMPTY_FIELD_DEFAULT_VALUE,
                '@QTABONOPREVISAO@' => !empty($programacaoFerias['QTABONOPREVISAO'])    ? $programacaoFerias['QTABONOPREVISAO'] : self::EMPTY_FIELD_DEFAULT_VALUE
            );
            
            $emailContent = strtr($baseContent, $contentReplace);
            
            
            $subjectReplace = array(
                '@NOME_VINCULO@' => $vinculo['NMVINCULOM']
            );
            
            $emailSubject = strtr($baseSubject, $subjectReplace);
            
            
            $emailParams = array(
                'templateName' => $templateName,
                'email' => $email,
                'subject' => $emailSubject,
                'nmdestinatario' => $nmdestinatario,
                'nmremetente' => $nmremetente,
                'nrorg' => $nrorg,
                'nrestruturam' => $vinculo['NRESTRUTLEGAL'],
                'content' => $emailContent,
                'attachment' => $attachment,
                'validaEmailTabelaOper' => $validaEmailTabelaOper
            );
            
            $emailsAEnviar[] = $emailParams;
        }
        
        // Envia os emails
        $emailsEnviados = $this->sendEmails($emailsAEnviar);
        
        return $emailsEnviados;
    }
    
    /**
     * Rotina de envio individual de email
     * 
     * @param string $email
     * @param string $subject
     * @param string $nmdestinatario
     * @param string $content
     * @param string $nrestruturam
     * @param string $nmremetente             Default: self::REMETENTE_DEFAULT
     * @param string $attachment              Default: null
     * @param bool   $validaEmailTabelaOper   Default: self::VALIDAEMAILTABELAOPER
     * @param string $templateName            Default: self::TEMPLATE_DEFAULT
     */
    public function sendEmail(
        string $email,
        string $subject,
        string $nmdestinatario,
        string $content,
        string $nrestruturam,
        string $nmremetente             = self::REMETENTE_DEFAULT,
        string $attachment              = null,
        bool   $validaEmailTabelaOper   = self::VALIDAEMAILTABELAOPER,
        string $templateName            = self::TEMPLATE_DEFAULT
    ) {
        $nrorg = $this->environment->getNrorg();
        
        $htmlBodyContent = array(
            "SUBJECT"                 => $subject,
            "STRING_SUPERIOR_DIREITA" => null,
            "CONTENT_BODY"            => $content  
        );
        
        return SendEmail::sendEmail(
            $templateName,
            $email,
            $subject,
            $nmdestinatario,
            $nmremetente,
            $nrorg,
            $nrestruturam,
            $htmlBodyContent,
            $attachment,
            $validaEmailTabelaOper
        );
    }
    
    /**
     * Rotina de envio de vários emails simultâneamente
     * 
     * @param Array EmailsParams $emailsParams
     * 
     * EmailsParams:
     *  string $email
     *  string $subject
     *  string $nmdestinatario
     *  string $content
     *  string $nrestruturam
     *  string $nmremetente             Default: self::REMETENTE_DEFAULT
     *  string $attachment              Default: null
     *  bool   $validaEmailTabelaOper   Default: self::VALIDAEMAILTABELAOPER
     *  string $templateName            Default: self::TEMPLATE_DEFAULT
     */
    public function sendEmails(Array $emailsParams) {
        foreach ($emailsParams as $emailParams) {
            
            // Default parameters
            $nmremetente            = !empty($emailParams['nmremetente'])           ? $emailParams['nmremetente']           : null;
            $attachment             = !empty($emailParams['attachment'])            ? $emailParams['attachment']            : null;
            $validaEmailTabelaOper  = !empty($emailParams['validaEmailTabelaOper']) ? $emailParams['validaEmailTabelaOper'] : null;
            $templateName           = !empty($emailParams['templateName'])          ? $emailParams['templateName']          : null;
            
            try {
                $emailEnviado = $this->sendEmail(
                    $emailParams['email'],
                    $emailParams['subject'],
                    $emailParams['nmdestinatario'],
                    $emailParams['content'],
                    $emailParams['nrestruturam'],
                    $emailParams['nmremetente'],
                    $emailParams['attachment'],
                    $emailParams['validaEmailTabelaOper'],
                    $emailParams['templateName']
                );
                
                if (!$emailEnviado) {
                    throw new \Exception(Messages::MAIL_NOT_SENT);
                }
            } catch (\Exception $e) {
                $emailsNaoEnviados[] = array('email' => $emailParams['email'], 'erro' => $e->getMessage());
            }
        }
        
        // Emails enviados com sucesso
        if (empty($emailsNaoEnviados)) {
            return true;
        }
        
        // Retorna emails nao enviados
        return $emailsNaoEnviados;
    }
    
    public function agrupaMensagensEmail($mensagensEmail) {
        if (empty($mensagensEmail)) {
            return false;
        }
        
        $mensagem = Messages::MAILS_NOT_SENT . '<br>';
        foreach ($mensagensEmail as $mensagemEmail) {
            $mensagem .= 'Email: ' . $mensagemEmail['email'] . '. Erro: ' . $mensagemEmail['erro'] . '<br>';
        }
        
        return $mensagem;
    }
    
    /**
     *  Função que retorna dados úteis do operador para as rotinas de EMAIL. 
     * 
     *  @param string $cdoperador
     * 
     *  @return Array $dadosOperador
     *      'NMOPERADOR'    => $nmoperadornotificacao,
     *      'EMAILOPEADOR'  => $emailoperadornotificacao
     * 
     *  @throws Exception
     */
    public function getDadosOperador($cdoperador, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosOperador = $this->operadorService->getDadosOperador($cdoperador);
        
        return $dadosOperador;
    }
    
    /**
     *  Função que retorna dados úteis do vínculo para as rotinas de EMAIL. 
     * 
     *  @param string $nrvinculom
     * 
     *  @return Array $dadosVinculo
     *      'NMVINCULOM'    => $nmvinculom
     * 
     *  @throws Exception
     */
    public function getDadosVinculo($nrvinculom, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosVinculo = $this->vinculoService->getDadosVinculo($nrvinculom, $nrorg);
        
        return $dadosVinculo;
    }
    
    /**
     *  Função que retorna dados úteis da estrutura para as rotinas de EMAIL. 
     * 
     *  @param string $nrvinculom
     * 
     *  @return Array $dadosVinculo
     *      'NMVINCULOM'    => $nmvinculom
     * 
     *  @throws Exception
     */
    public function getDadosEstrutura($nrestruturam, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }
        
        $dadosEstrutura = $this->estruturaService->getDadosEstrutura($nrestruturam, $nrorg);
        
        return $dadosEstrutura;
    }
    
    
    /**
    *   Função usada para enviar email de notificação de solicitação da tela de Minha história 
    *
    * 
    **/
    
    public function notificaMinhaHistoria(
                $nrvinculom,
        int     $nrSolcitacao,
        string  $attachmentFile = null
    ){
        
        $nrorg = $this->environment->getNrorg();
        
        $vinculo = $this->getDadosVinculo($nrvinculom);
        $unidade = $this->getDadosEstrutura($vinculo['NRESTRUTGEREN'], $nrorg);
        $nmgerem = $unidade['NMESTRUTURAM'];
        $nrEstrutLegal = $vinculo['NRESTRUTLEGAL']; 
        $nmvinculo = $vinculo['NMVINCULOM'];
        list ($dia, $mes, $ano) = explode ('/', date("d/m/Y"));
        $dtmescompetenc = "01/".$mes."/".$ano;
        $operadores = $this->operadorService->getOperadoresDp();
        
        $dados = $this->estruturaService->retornaDadosResponsavelInformal($nrorg, $unidade['NRESTRUTURAM'], $dtmescompetenc);
        
        if(empty($dados)){
            $emailResp = "E-mail do responsável pela estrutura gerencial ".$nmgerem." não cadastrado";
            $nmdestinatarioResp ="Responsável pela estrutura gerencial ".$nmgerem." não cadastrado";
        }else {
            $emailResp = $dados['CDCOMUNICAPARC'];
            $nmdestinatarioResp = $dados['NMPESSOA'];
        }
 
        $subject = "Notificação Solicitação | Vínculo: " . $nmvinculo. ", Solicitação: " . $nrSolcitacao;
        
        // Busca itens da solicitação
        $itensSolicitacao = $this->solicitacaoService->getItemSolicitacao($nrSolcitacao);
        
        // Busca detalhes da solicitação
        $solicitacao = $this->solicitacaoService->getSolicitacao($nrSolcitacao);
        if (empty($solicitacao)) {
            throw new \Exception('A solicitação não foi encontrada no momento do disparo de emails.');
        }
        
        // Mensagem dos itens
        $mensagemBaseItens = '
            <p style="font-size: 1.2rem;"><b>Itens</b></p>
            <table style="width:100%">
                <tr>
                    <th>Campo</th>
                    <th>Valor</th>
                </tr>
                @SOLICITACAO_ITENS@
            </table>
            <p>&nbsp;</p>
        ';
        
        $mensagemItens = '';
        foreach ($itensSolicitacao as $item) {
            $mensagemItens .=  "<tr><td><p>${item['DSLABELITEMSOLI']}</p></td><td><p>${item['DSVALORITEMSOLI']}</p></td></tr>";
        }
        
        // Monta mensagem final dos itens
        $mensagemFinalItens = !empty($mensagemItens) ? strtr($mensagemBaseItens, array('@SOLICITACAO_ITENS@' => $mensagemItens)) : '';
        
        // Corpo mensagem
        $mensagem = '
        <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee">
           <tbody>
              <tr>
                 <td align="center">
                    <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color">
                       <tbody>
                          <tr>
                             <td>
                                <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580">
                                   <tbody>
                                      <tr>
                                         <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: \'Questrial\', sans-serif; mso-line-height-rule: exactly; padding-bottom:15px;">
                                            <div>
                                               <p>&nbsp;</p>
                                               
                                               <p style="font-size: 1.3rem;"><font><strong>'. $solicitacao['NMTIPOSOLICITACAO'] .'</strong></font></p>
                                               
                                               <p>&nbsp;</p>
                                               
                                               <table style="width:100%">
                                                    <tr><td>Unidade</td><td>'.$nmgerem.'</td></tr>
                                                    <tr><td>Responsável</td><td>'.$nmdestinatarioResp.'</td></tr>
                                                    <tr><td>E-mail Responsável</td><td>'.$emailResp.'</td></tr>
                                                    <tr><td>Vínculo</td><td>'.$nmvinculo.'</td></tr>
                                                    <tr><td>Nr. Solicitação</td><td>'. $nrSolcitacao . '</td></tr>
                                              </table>
                                              
                                              <p>&nbsp;</p>
                                               
                                               ' . $mensagemFinalItens .'
                                               <p>O documento associado à solcitação segue em anexo ao email.</p>
                                            </div>
                                         </td>
                                      </tr>
                                      <tr>
                                         <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td>
                                      </tr>
                                   </tbody>
                                </table>
                             </td>
                          </tr>
                       </tbody>
                    </table>
        ';
        
        foreach ($operadores as $key => $value){
            $nmOperadorDp = $value['NMOPERADOR'];
            $emailOperadorDp = $value['DSEMAILOPER'];
            
            $emailEnviado = $this->sendEmail(
                $emailOperadorDp,
                $subject,
                $nmOperadorDp,
                $mensagem,
                $nrEstrutLegal,
                self::REMETENTE_DEFAULT,
                $attachmentFile,
                false,
                self::TEMPLATE_DEFAULT
            );
        }
    }
}