<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Ems\Services\EmailService;
use Util\Environment;
use Ems\Repository\VinculoRepository;
use Util\NovoCodigo;

use Ems\Repository\MinhaHistoriaRepository;
use Ems\Util\{ Utilities, ArrayUtil };

class MinhaHistoriaService {

    private $default_key = ['NRORG','NRVINCULOM','NRTIPOSOLICITACAO','DSOBSSOLICITACAO', 'DTSOLICITACAO'];

    private $labelFieldPessoa = [
        "NRORG"             => "Organização",
        "DTMESCOMPETENC"    => "Competência",
        "DTESOCIAL"         => "Data Esocia",
        "NRPESSOA"          => "Nr Pessoa",
        "NRPARCNEGOCIO"     => "Nr Parceiro Negocio",
        "NRPESSOAH"         => "Nr Historico",
        "DSPRONOMETRAT"     => "Tratamento",
        "NMPESSOA"          => "Nome",
        "DSAPELIDOPESS"     => "Apelido",
        "DTNASCPESSOA"      => "Dt. Nasc.",
        "IDSEXOPESSOA"      => "Sexo",
        "NMESTACIVIL"       => "Estado Civil",
        "CDESTACIVIL"       => "CD. Estado Civil",
        "IDGRUPSANGUPES"    => "Grupo Sanguíneo",
        "IDFATORRHPES"      => "Fator RH",
        "DSCONDFISPES"      => "Condição Física",
        "NRCONDFISPES"      => "Nr. Condição Física",
        "IDCOTAPCD"         => "Cota PCD",
        "NMRACAPESSOA"      => "Raça",
        "NRRACAPESSOA"      => "Nr. Raça",
        "NMGRAUINSTR"       => "Grau de Instrução",
        "NRGRAUINSTR"       => "Nr. Grau de Instrução",
        "NMPAIS"            => "País",
        "CDMUNICIPIO"       => "Cd. Município",
        "NMMUNICIPIO"       => "Município",
        "NRNACIONALIDADE"   => "Nr. Nacionalidade",
        "DSNACIONALIDADE"   => "Nacionalidade",
        "DTADMISSAOPRE"     => "Dt. Admissão Preliminar",
        "CDRECIBOADMPRE"    => "Nº Recibo Admissão Preliminar",
        "DSEXPDOCESTR"      => "Órgão emissor da RNE",
        "DSCLASSESTRANG"    => "Classificação Trabalhador Estrangeiro",
        "NRCLASSESTRANG"    => "Nr. Classificação Trabalhador Estrangeiro",
        "DTCHEGADAPAIS"     => "Dt. Chegada País",
        "NRDOCUESTRANG"     => "Doc. Estrangeiro",
        "IDPESNATURALIZA"   => "Naturalizado",
        "DTNATURALIZAPES"   => "Dt. Naturalidade",
        "IDPOSVISTOPERM"    => "Visto Permanente",
        "DTPERMAVISTO"      => "Dt. Permanência",
        "NRCPFPESSOA"       => "CPF",
        "NRRGPESSOA"        => "RG",
        "CDEXRGPESSOA"      => "Cd. Órgão Expedidor RG",
        "NMEXRGPESSOA"      => "Órgão Expedidor RG",
        "DTEXRGPESSOA"      => "Dt. Expedição RG",
        "DSLOCALEXRGPES"    => "Local de Expedição",
        "SGUFRGPESSOA"      => "UF-RG",
        "NRSUSPESSOA"       => "Nr. Cartão SUS",
        "NRNITPESSOA"       => "NIT",
        "NRTITUELEIPES"     => "Título de Eleitor",
        "NRZONAELEIPES"     => "Zona Eleitoral",
        "NRSECAELEIPES"     => "Seção Eleitoral",
        "IDISENTITUELEI"    => "Isento Título Eleitoral",
        "NRCERTNASCPES"     => "Certidão Nascimento",
        "NRLIVROCERTNASC"   => "Livro Certidão Nascimento",
        "NRFOLHACERTNASC"   => "Folha Certidão Nascimento",
        "DSCOMPLCERTNASC"   => "Complemento Certidão Nasc.",
        "NRCERTOBITO"       => "Nr. certificado de obito",
        "NRCERTCASAPES"     => "Certidão Casamento",
        "DTCASAPESSOA"      => "Dt. Casamento",
        "NRCARTHABPES"      => "CNH",
        "DTHABCARTPES"      => "Dt. Habilitação",
        "DSCATEGHABCART"    => "Categoria CNH",
        "DTVALHABCARTPES"   => "Validade CNH",
        "SGUFHABCARPES"     => "UF CNH",
        "NRCTPSPESSOA"      => "Nr. CTPS",
        "NRSERIECTPSPES"    => "Série CTPS",
        "SGUFCTPSPES"       => "UF do CTPS",
        "DTCTPSPESSOA"      => "Dt. Emissão CTPS",
        "NRPISPASEPPES"     => "Nr. PIS/PASEP",
        "DTINSCPISPASEP"    => "Dt. PIS/PASEP",
        "NRINSCINSSPES"     => "INSS",
        "NRINSCISSPES"      => "ISS",
        "NRREGCTPSPES"      => "Registro Profissional MTE",
        "NMCONSELHOREG"     => "Conselho Regional",
        "NRCONSELHOREG"     => "Nr. Conselho Regional",
        "NRINSCRCONSREG"    => "Inscrição no Conselho",
        "DTEXCONSREG"       => "Dt. Ex. Conselho",
        "NMUFCONSREG"       => "UF Conselho",
        "CDPAIS"            => "Cd. País",
        "SGESTADO"          => "Sg. Estado",
        "NMESTADO"          => "Estado",
        "DTINIVIGENCIA"     => "Início Vigência Conselho",
        "DTFIMVIGENCIA"     => "Fim Vigência Conselho",
        "NRCATMILITARPE"    => "Nr. Categoria Militar",
        "NMCATMILITARPE"    => "Categoria Militar",
        "NRCERTRESEPES"     => "Certificado de Reservista",
        "CDEXCERTRESEPES"   => "Cd. Expedição Certificado de Reservista",
        "DSCATEGCERTRESE"   => "Categ. Certificado de Reservista",
        "DTCERTRESEPES"     => "Dt. Certificado de Reservista",
        "CDBANCO"           => "Nr. Banco",
        "NMBANCO"           => "Nm. Banco",
        "CDAGENCIA"         => "Cd. Agencia Bancaria",
        "NMAGENCIA"         => "Nm. Agencia Bancaria",
        "NRTPCONTCORR"      => "Nr. Tipo Conta Corrente",
        "DSTPCONTCORR"      => "Tipo Conta Corrente",
        "CDCONTCORR"        => "Conta Corrente",
        "NRCEPENDERECO"     => "Número do CEP",
        "CDTIPOENDERECO"    => "CD. Tipo Endereço",
        "NMTIPOENDERECO"    => "Nm. Tipo Endereço",
        "NMBAIRROENDERECO"  => "Bairro",
        "CDLOGRADOURO"      => "Cd. Logradouro",
        "DSLOGRADOURO"      => "Ds. Logradouro",
        "DSENDERECO"        => "Endereço",
        "NRIMOVELENDERECO"  => "Nr Imovel",
        "DSCOMPLEENDERECO"  => "Complemento Imovel",
        "DSREFERENCIAENDE"  => "Referencia Imovel",
        "NRTPAVIPRE"        => "Tipo de Aviso Prévio",
        "NRTPDEMISSAO"      => "Tipo de Demissão",
        "NRMOTIVORESC"      => "Motivo da Rescisão",
        "DTAVISOPREVIO"     => "Dt de aviso prévio",
        "DTRESCISAOVINC"    => "Dt de Rescisão Vínculo",
        "VRSALARIO"         => "Valor Salário",
        "NRTIPOALTERA"      => "Tipo de Alteração Salarial",
        "NROCORRENCIA"      => "Nr. Ocorrência",
        "IDTPSALARIO"       => "Tipo de Salário",
        "NRTPMODALIDSAL"    => "Tipo de Modalidade do Salário",
        "DTALTESALARIO"     => "Dt de Alteração Salarial",
        "DSOBSERVACAO"      => "Observação",
        "DTINIOCUPACAO"     => "Dt. Início Sit. Funcional",
        "NROCUPACAOM"       => "Nr. Ocupação",
        "NRMOTIVOOCUPA"     => "Nr. Motivo Ocupação",
        "NRTIPOOCUPACAO"    => "Nr. Tipo Ocupação",
        "DTINIESCALA"       => "Início da Escala",
        "NRTURNO"           => "Turno",
        "NRESCALATRABM"     => "Escala",
        "NRTIPORELACIONA"   => "Tipo de relacionamento",
        "NMPESSOADEP"       => "Nome",
        "DTNASCIFUNDPARC"   => "Data de nascimento",
        "IDSEXOPESSOA"      => "Sexo",
        "NRCPFDEP"          => "CPF",
        "NRLINHATRANSP"     => "Linha",
        "NRTIPOLINHAVINC"   => "Tipo da Linha",
        "NRSENTIDOLINHA"    => "Sentido da Linha",
        "QTLINHATARIFA"     => "Quantidade",
        "DTINILINHAVINC"    => "Inicio de Validade",
        "DTFIMLINHAVINC"    => "Fim de Validade",
        "NRBENEFICIOM"      => "Benefício",
        "DTINSCBENEFICIARIO"=> "Data Inscrição",
        "NRPESSOA"          => "Pessoa",
        "IDTPBENEFICIARIO"  => "Tipo Beneficiário",
        "NRTEMPLATE" => "Template",
        "NRTIPOPROCESSO" => "Tipo do Processo",
        "NRPARCNEGOCIORESP" => "Responsável",
        "DSMOTIVO" => "Motivo",
        "LOCALIDADE" => "Localidade",
        "DSDESCRICAO" => "Descrição",
        "DTSOLICITA" => "Disponível desde",
        "DTLIMSOLICITA" => "Disponível até",
        "NROCUPACAOM" => "Ocupação",
        "NRESTRUTGEREN" => "Estrutura Gerencial",
        "NRVAGAS" => "Nº de Vagas",
        "IDPUBLICAVAGA" => "Publicar Vaga",
        "NMPROCESSO" => "Nome do Processo",
    ];

    private $labelFieldComunica = [ "PREFTELPRINCIPAL"  => "Prefixo Telefone principal",
                                    "TELPRINCIPAL"      => "Telefone principal",
                                    "PREFTELRESIDEN"    => "Prefixo Telefone residencial",
                                    "TELRESIDEN"        => "Telefone residencial",
                                    "PREFTELCOMERC"     => "Prefixo Telefone comercial",
                                    "TELCOMERC"         => "Telefone comercial",
                                    "PREFCELPARTIC"     => "Prefixo Celular particular",
                                    "CELPARTIC"         => "Celular particular",
                                    "PREFCELCOMERC"     => "Prefixo Celular comercial",
                                    "CELCOMERC"         => "Celular comercial",
                                    "PREFFAX"           => "Prefixo Fax",
                                    "FAX"               => "Fax",
                                    "EMAIL"             => "Email",
                                    "CDFORMACOMU"       => "Cd. Forma Comunicação"
    ];
                                    
    private const SOLICITACAO_ALTERACAO_RECEBE_ANEXO = true;
    
    private const TIPO_SOLICITACAO_CAMPOS = [
        // Pessoal
        6 => [
            'NRTEMPLATE',
            'NRTIPOPROCESSO',
            'NRPARCNEGOCIORESP',
            'DSMOTIVO',
            'LOCALIDADE',
            'DSDESCRICAO',
            'DTSOLICITA',
            'DTLIMSOLICITA',
            'NROCUPACAOM',
            'NRESTRUTGEREN',
            'NRVAGAS',
            'IDPUBLICAVAGA',
            'NMPROCESSO',
        ],
        // Meus dados
        7 => [
            'NRCERTCASAPES',
            'NRGRAUINSTR',
            'DSPESSOA',
            'NMPESSOA',
            'NRCARTHABPES',
            'DSCATEGHABCART',
            'DTCASAPESSOA',
            'DTHABCARTPES',
            'DTVALHABCARTPES',
        ],
        // Endereço
        8 => [
            'NRCEPENDERECO',
            'CDTIPOENDERECO',
            'CDPAIS',
            'SGESTADO',
            'CDMUNICIPIO',
            'NMBAIRROENDERECO',
            'CDLOGRADOURO',
            'DSENDERECO',
            'NRIMOVELENDERECO',
            'DSCOMPLEENDERECO',
            'DSREFERENCIAENDE',
        ],
        // VT
        10 => [
            "NRLINHATRANSP",
            "NRTIPOLINHAVINC",
            "NRSENTIDOLINHA",
            "QTLINHATARIFA",
            "DTINILINHAVINC",
            "DTFIMLINHAVINC",
        ],
        // Dependenete
        11 => [
            "NRTIPORELACIONA",
            "NMPESSOADEP",
            "DTNASCIFUNDPARC",
            "IDSEXOPESSOA",
            "NRCPFDEP",
        ],
        // Benefício
        13 => [
            "NRBENEFICIOM",
            "DTINSCBENEFICIARIO",
            "NRPESSOA",
            "IDTPBENEFICIARIO",
        ],
        15 => [
            'CDBANCO',
            'CDAGENCIA',
            'CDCONTCORR',
            'NRTPCONTCORR',
        ],
    ];
    
    const CDFORMACOMU_TELEFONE_PRINCIPAL    = '01';
    const CDFORMACOMU_CELULAR_PARTICULAR    = '02';
    const CDFORMACOMU_TELEFONE_RESIDENCIAL  = '03';
    const CDFORMACOMU_TELEFONE_COMERCIAL    = '04';
    const CDFORMACOMU_EMAIL                 = '05';
    const CDFORMACOMU_CELULAR_COMERCIAL     = '06';
    const CDFORMACOMU_FAX                   = '10';

    
    public function __construct(
        MinhaHistoriaRepository $minhaHistoriaRepository,
        VinculoRepository $vinculoRepository,
        Environment $environment,
        EmailService $emailService
    ) {
        $this->minhaHistoriaRepository = $minhaHistoriaRepository;
        $this->vinculoRepository       = $vinculoRepository;
        $this->environment             = $environment;
        $this->emailService             = $emailService;
    }



    public function solicitaAlteracao($row) {
    try {
            
            $nrorg      = $row['NRORG'];
            $nrvinculom = $row['NRVINCULOM'];
            $tipoSoli   = $row['NRTIPOSOLICITACAO'];
            $filePath   = null;
            
            // Anexo ao email
            if (isset($row['DSFILE']) && isset($row['NMFILE'])) {
                $fileContent    = $row['DSFILE'];
                $fileName       = $row['NMFILE'];
                unset($row['DSFILE']);
                unset($row['NMFILE']);
                
                $filePath = Utilities::blobToFile($fileContent, $fileName);
            }
            
            if(isset($row['DSOBSSOLICITACAO']))
                $dsobservacao = $row['DSOBSSOLICITACAO'];
            else
                $dsobservacao = null;
            
            $cdoperador = $this->vinculoRepository->getOperadorVinculo($nrorg, $nrvinculom);
            $cdoperador = $cdoperador['CDOPERADOR'];
            
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $itens = $this->validateTipoSolicitacaoCampos($tipoSoli, $row);
            
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            
            foreach ($itens as $key => $field) {
                if(!in_array($key, $this->default_key)){
                    $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                    $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, $key, $this->labelFieldPessoa[$key], strval($field), $nrorg, $cdoperador);
                }
            }
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function solicitaAlteracaoComunica($row){
        $nrorg      = $row['NRORG'];
        $nrvinculom = $row['NRVINCULOM'];
        $tipoSoli   = $row['NRTIPOSOLICITACAO'];
        if(isset($row['DSOBSSOLICITACAO']))
            $dsobservacao = $row['DSOBSSOLICITACAO'];
        else
            $dsobservacao = null;

        $cdoperador = $this->vinculoRepository->getOperadorVinculo($nrorg, $nrvinculom);
        $cdoperador = $cdoperador['CDOPERADOR'];

        $fileContent    = $row['DSFILE'];
        $fileName       = $row['NMFILE'];
        unset($row['DSFILE']);
        unset($row['NMFILE']);

        $filePath       = Utilities::blobToFile($fileContent, $fileName);
        
        if(isset($row['PREFTELPRINCIPAL']) || isset($row['TELPRINCIPAL'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELPRINCIPAL'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELPRINCIPAL'], $row['PREFTELPRINCIPAL'], $nrorg, $cdoperador);
            }
            if(isset($row['TELPRINCIPAL'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELPRINCIPAL'], $row['TELPRINCIPAL'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_PRINCIPAL, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['PREFTELRESIDEN']) || isset($row['TELRESIDEN'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELRESIDEN'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELRESIDEN'], $row['PREFTELRESIDEN'], $nrorg, $cdoperador);
            }
            if(isset($row['TELRESIDEN'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELRESIDEN'], $row['TELRESIDEN'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_RESIDENCIAL, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['PREFTELCOMERC']) || isset($row['TELCOMERC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFTELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFTELCOMERC'], $row['PREFTELCOMERC'], $nrorg, $cdoperador);
            }
            if(isset($row['TELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['TELCOMERC'], $row['TELCOMERC'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_TELEFONE_COMERCIAL, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['PREFCELPARTIC']) || isset($row['CELPARTIC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFCELPARTIC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFCELPARTIC'], $row['PREFCELPARTIC'], $nrorg, $cdoperador);
            }
            if(isset($row['CELPARTIC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['CELPARTIC'], $row['CELPARTIC'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_CELULAR_PARTICULAR, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['PREFCELCOMERC']) || isset($row['CELCOMERC'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFCELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFCELCOMERC'], $row['PREFCELCOMERC'], $nrorg, $cdoperador);
            }
            if(isset($row['CELCOMERC'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['CELCOMERC'], $row['CELCOMERC'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_CELULAR_COMERCIAL, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['PREFFAX']) || isset($row['FAX'])){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            if(isset($row['PREFFAX'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDPREFIXCOMUPARC', $this->labelFieldComunica['PREFFAX'], $row['PREFFAX'], $nrorg, $cdoperador);
            }
            if(isset($row['FAX'])){
                $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
                $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['FAX'], $row['FAX'], $nrorg, $cdoperador);
            }
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_FAX, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
        if(isset($row['EMAIL']) ){
            $nrsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_SOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereSolicitacao($nrsolicitacao, $tipoSoli, $nrvinculom, $nrorg, $cdoperador, $dsobservacao);
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDCOMUNICAPARC', $this->labelFieldComunica['EMAIL'], $row['EMAIL'], $nrorg, $cdoperador);
            
            $nritemsolicitacao = NovoCodigo::geraCodigo($nrorg, 'GPE_ITEMSOLICITACAO', 1, 12, true);
            $this->minhaHistoriaRepository->insereItemSolicitacao($nritemsolicitacao, $nrsolicitacao, 'CDFORMACOMU', $this->labelFieldComunica['CDFORMACOMU'], self::CDFORMACOMU_EMAIL, $nrorg, $cdoperador);
            
            // Notifica via email
            $this->emailService->notificaMinhaHistoria($nrvinculom, $nrsolicitacao, $filePath);
        }
        
    }

    /**
     *  Rotina responsável por filtrar (sanitize) os campos necessários para a criação de cada solicitação, por tipo. Campos com valor nulo não são retornados,
     * visto que a coluna de valor dos itens na tabela de itens não pode receber nulo
     * 
     *  @param  string      $tipoSoli
     *  @param  Array/Row   $itensSolicitacao
     * 
     *  @return Array       $itensSanitized
     * 
     */
    public function validateTipoSolicitacaoCampos($tipoSoli, $itensSolicitacao) {
        $camposItem = isset(self::TIPO_SOLICITACAO_CAMPOS[$tipoSoli]) ? self::TIPO_SOLICITACAO_CAMPOS[$tipoSoli] : false;
        if (empty($camposItem)) {
            return [];
        }
        
        $itens = !is_array($itensSolicitacao) ? (array) $itensSolicitacao : $itensSolicitacao;
        $itensSanitized = array_filter($itens, function($itemKey) use ($camposItem, $itens) {
            return in_array($itemKey, $camposItem) && !is_null($itens[$itemKey]);
        }, ARRAY_FILTER_USE_KEY);
        
        return $itensSanitized;
    }
    
}
