/******************************************************************************/
/* Sistemas Integrados TEKNISA                                                */
/* Nome: 0001_CARSTPO_USR_EXPORTASOLICITA_COMPASSARGENTINA.sql                */
/*                                                                            */
/* Descrio:                                                                 */
/* Procedure destinada a possibilitar que as solicitaes de compras          */
/* geradas pelo sistema da Teknisa sejam importadas                           */
/* no Portal da Compass Argentina e, para isso, a procedure alimentar        */
/* duas tabelas intermedirias entre os dois sistemas (tabelas pertencentes   */
/* ao banco de dados do sistema da Teknisa).                                  */
/* Criao:                                                                   */
/* 18/01/2012 - Marcelo Alexsandro                                            */
/******************************************************************************/

CREATE OR REPLACE
PROCEDURE USR_EXPORTASOLICITA IS
   V_DTGRV         DATE;
   V_HRGRV         VARCHAR2(4);
   V_NRSEQ         VARCHAR(4);
   V_NRSEQUENCIAL  INTEGER;
   V_OLDNRSEQ      INTEGER;
   V_NOVOCODIGO    INTEGER;
   V_PRXCODIGO     VARCHAR2(20);
   V_CHAVESOLANT   VARCHAR2(18);
   V_CUSTOLISPREC  FLOAT;
   V_CDFORNECED    VARCHAR2(21);
   V_CDPERIENTR    VARCHAR2(2);
   V_NMPERIENTR    VARCHAR2(20);
   V_NOVADTENTREGA DATE;
   V_DIAS          VARCHAR2(300);
   V_IDTIPODIA     PERIENTR.IDTIPODIA%TYPE;
   V_RETURN        VARCHAR2(1);
   V_ERROR VARCHAR2(4000);
BEGIN
   V_RETURN := USR_FUNC_BLOQUEIO('USR_INT_ITEMSOLI', 'B');
   IF V_RETURN = 'S' THEN
      V_DTGRV := NULL;
      V_HRGRV := NULL;

      V_NRSEQ := '0001';

      /* Retorna data e hora de exportao */
      BEGIN
        SELECT TO_DATE(SYSDATE), TO_CHAR(SYSDATE,'HH24MI')
          INTO V_DTGRV, V_HRGRV
          FROM DUAL;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           V_DTGRV := NULL;
           V_HRGRV := NULL;
      END;
      V_CHAVESOLANT := NULL;
      FOR C_SOLICITA IN
        (SELECT
            I.CDFILSOLI,  I.CDFILLAN CDFILLANCA, I.NRSOLICMP,
            I.IDITMSOLEXTRA IDSOLEXTRA, I.CDPRODUTO, PRODUTO.CDARVPROD, I.DTPROGENTR,
            I.QTSOLIPROD, I.DTUTILIZA, I.TXOBSITMSOL, PRODUTO.CDPRODESTO, NVL(PRODUTO.VRFATOCONV, 1) VRFATOCONV,
            PRODUTO.NMPRODUTO, I.USR_CDORIGEMPORTAL CDORIGEMPORTAL, I.USR_NRORIGEMPORTAL NRORIGEMPORTAL, I.USR_NRITEMPORTAL NRITEMPORTAL
           FROM ITEMSOLI I, PRODUTO
          WHERE I.USR_DTEXPORTA IS NULL
            AND I.CDPRODUTO = PRODUTO.CDPRODUTO
            AND I.IDSOLIAPROV = 'S'
            AND EXISTS (SELECT 1
                          FROM GRUPFILI, FILGRUPFILI
                         WHERE GRUPFILI.CDGRUPFILI = FILGRUPFILI.CDGRUPFILI
                           AND GRUPFILI.CDGRUPFILI = '9999999999'
                           AND FILGRUPFILI.CDFILIAL = I.CDFILSOLI)
          ORDER BY
             I.CDFILSOLI,
             I.CDFILLAN,
             I.NRSOLICMP,
             I.CDPRODUTO,
             I.DTPROGENTR)
      LOOP
        /* Verifica existncia do item de solicitao de compras */
        SELECT MAX(NRSEQUENCIAL)
          INTO V_NRSEQUENCIAL
          FROM USR_INT_ITEMSOLI
         WHERE CHAVETEKNISA = C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP
           AND CDPRODUTO = C_SOLICITA.CDPRODUTO
           AND DTPROGENTR = C_SOLICITA.DTPROGENTR
           AND IDTPEXPITSOL = '1'
           AND IDORIGREGISTRO = 'T';
        IF (V_CHAVESOLANT IS NULL) OR (V_CHAVESOLANT <> C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP) THEN
           IF V_NOVOCODIGO IS NOT NULL THEN
              UPDATE USR_INT_ITEMSOLI
                 SET DTGRAVACAO = V_DTGRV, HRGRAVACAO = V_HRGRV
               WHERE USR_INT_ITEMSOLI.NRSEQUENCIAL = V_NOVOCODIGO;
               COMMIT;
           END IF;

           NOVO_CODIGO('USR_INT_ITEMSOLI', V_PRXCODIGO);
           V_NOVOCODIGO := TO_NUMBER(SUBSTR(V_PRXCODIGO,11,10));
           V_CHAVESOLANT := C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP;
           V_NRSEQ := '0001';
        END IF;

        V_CUSTOLISPREC := (CUSTO_PRODUTO(C_SOLICITA.CDFILSOLI, C_SOLICITA.CDPRODESTO, 6, C_SOLICITA.DTPROGENTR, SYSDATE))* C_SOLICITA.VRFATOCONV;
        V_CDFORNECED   := USR_FORN_LISTAPRECO(C_SOLICITA.CDFILSOLI, C_SOLICITA.CDPRODESTO, C_SOLICITA.DTPROGENTR);

        V_CDPERIENTR := NULL;
        V_NMPERIENTR := NULL;
        SELECT MAX(P.CDPERIENTR), MAX(P.NMPERIENTR), MAX(IDTIPODIA)
          INTO V_CDPERIENTR, V_NMPERIENTR, V_IDTIPODIA
          FROM REFORFIL, PERIENTR P
         WHERE REFORFIL.CDFORNECED = V_CDFORNECED
           AND REFORFIL.CDFILIAL = C_SOLICITA.CDFILSOLI
           AND REFORFIL.CDPERIENTR = P.CDPERIENTR;

        V_NOVADTENTREGA := C_SOLICITA.DTPROGENTR;
        V_DIAS          := NULL;
        IF V_CDPERIENTR IS NOT NULL THEN
           V_NOVADTENTREGA := CARREGA_PERIFORN_FUN(C_SOLICITA.CDFILSOLI, V_CDFORNECED, C_SOLICITA.DTPROGENTR);
           FOR C_DIAS IN
              (SELECT DIENTREGA
                 FROM DIASENTR
                WHERE CDPERIENTR = V_CDPERIENTR)
           LOOP
              V_DIAS := V_DIAS||', '||C_DIAS.DIENTREGA;
           END LOOP;

           IF V_IDTIPODIA = 'S' THEN
              V_DIAS := 'SEMANALMENTE - '||V_DIAS;
           ELSE
              IF V_IDTIPODIA = 'M' THEN
                 V_DIAS := 'MENSUAL - '||V_DIAS;
              ELSE
                 V_DIAS := 'SUPLICANTE SEMANALMENTE - '||V_DIAS;
              END IF;
           END IF;
        END IF;

        IF V_NRSEQUENCIAL IS NULL THEN
           INSERT INTO USR_INT_ITEMSOLI
              (NRSEQUENCIAL, CDFILSOLI, CHAVETEKNISA,
               IDSOLEXTRA, NRSEQITEM, CDPRODUTO, DTPROGENTR,
               QTSOLIPROD, DTUTILIZA, TXOBSITMSOL, CDFORNECED, DTENTREGAFORN, VRCUSTOLISPREC, CDPERIFORN, NMPERIFORN,
               IDTPEXPITSOL, IDORIGREGISTRO, DTGRAVACAO, HRGRAVACAO,
               DTIMPORTACAO, HRIMPORTACAO, NMPRODUTO, DIAS, CDORIGEMPORTAL, NRORIGEMPORTAL, NRITEMPORTAL
               )
           VALUES
              (V_NOVOCODIGO, C_SOLICITA.CDFILSOLI, C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP,
               C_SOLICITA.IDSOLEXTRA, V_NRSEQ, C_SOLICITA.CDARVPROD, C_SOLICITA.DTPROGENTR,
               C_SOLICITA.QTSOLIPROD, C_SOLICITA.DTUTILIZA, C_SOLICITA.TXOBSITMSOL, V_CDFORNECED, V_NOVADTENTREGA, V_CUSTOLISPREC, V_CDPERIENTR, V_NMPERIENTR,
               '1', 'T', NULL, NULL,
               NULL, NULL, C_SOLICITA.NMPRODUTO, V_DIAS, C_SOLICITA.CDORIGEMPORTAL, C_SOLICITA.NRORIGEMPORTAL, C_SOLICITA.NRITEMPORTAL);
        ELSE
           FOR C_ITEM IN
             (SELECT NRSEQITEM, CDPRODUTO, DTPROGENTR, QTSOLIPROD, DTUTILIZA, TXOBSITMSOL, CDFORNECED, DTENTREGAFORN, VRCUSTOLISPREC, CDPERIFORN, NMPERIFORN, NMPRODUTO, DIAS, CDORIGEMPORTAL, NRORIGEMPORTAL, NRITEMPORTAL
                FROM USR_INT_ITEMSOLI
               WHERE CHAVETEKNISA = C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP
                 AND CDPRODUTO    = C_SOLICITA.CDARVPROD
                 AND DTPROGENTR   = C_SOLICITA.DTPROGENTR
                 AND NRSEQUENCIAL = V_NRSEQUENCIAL)
           LOOP
              INSERT INTO USR_INT_ITEMSOLI
                 (NRSEQUENCIAL, CDFILSOLI, CHAVETEKNISA,
                  IDSOLEXTRA, NRSEQITEM, CDPRODUTO, DTPROGENTR,
                  QTSOLIPROD, DTUTILIZA, TXOBSITMSOL, CDFORNECED, DTENTREGAFORN, VRCUSTOLISPREC, CDPERIFORN, NMPERIFORN,
                  IDTPEXPITSOL, IDORIGREGISTRO, DTGRAVACAO, HRGRAVACAO,
                  DTIMPORTACAO, HRIMPORTACAO, NMPRODUTO, DIAS, CDORIGEMPORTAL, NRORIGEMPORTAL, NRITEMPORTAL)
              VALUES
                 (V_NOVOCODIGO, C_SOLICITA.CDFILSOLI, C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP,
                  C_SOLICITA.IDSOLEXTRA, C_ITEM.NRSEQITEM, C_ITEM.CDPRODUTO, C_ITEM.DTPROGENTR,
                  C_ITEM.QTSOLIPROD, C_ITEM.DTUTILIZA, C_ITEM.TXOBSITMSOL, C_ITEM.CDFORNECED, C_ITEM.DTENTREGAFORN, C_ITEM.VRCUSTOLISPREC, C_ITEM.CDPERIFORN, C_ITEM.NMPERIFORN,
                  '2', 'T', NULL, NULL,
                  NULL, NULL, C_ITEM.NMPRODUTO, C_ITEM.DIAS, C_ITEM.CDORIGEMPORTAL, C_ITEM.NRORIGEMPORTAL, C_ITEM.NRITEMPORTAL);
           END LOOP;

           UPDATE USR_INT_ITEMSOLI
             SET DTGRAVACAO = V_DTGRV, HRGRAVACAO = V_HRGRV
           WHERE USR_INT_ITEMSOLI.NRSEQUENCIAL = V_NOVOCODIGO;
           COMMIT;

           NOVO_CODIGO('USR_INT_ITEMSOLI', V_PRXCODIGO);
           V_NOVOCODIGO := TO_NUMBER(SUBSTR(V_PRXCODIGO,11,10));

           INSERT INTO USR_INT_ITEMSOLI
              (NRSEQUENCIAL, CDFILSOLI, CHAVETEKNISA,
               IDSOLEXTRA, NRSEQITEM, CDPRODUTO, DTPROGENTR,
               QTSOLIPROD, DTUTILIZA, TXOBSITMSOL, CDFORNECED, DTENTREGAFORN, VRCUSTOLISPREC, CDPERIFORN, NMPERIFORN,
               IDTPEXPITSOL, IDORIGREGISTRO, DTGRAVACAO, HRGRAVACAO,
               DTIMPORTACAO, HRIMPORTACAO, NMPRODUTO, DIAS, CDORIGEMPORTAL, NRORIGEMPORTAL, NRITEMPORTAL)
           VALUES
              (V_NOVOCODIGO, C_SOLICITA.CDFILSOLI, C_SOLICITA.CDFILSOLI||C_SOLICITA.CDFILLANCA||C_SOLICITA.NRSOLICMP,
               C_SOLICITA.IDSOLEXTRA, V_NRSEQ, C_SOLICITA.CDARVPROD, C_SOLICITA.DTPROGENTR,
               C_SOLICITA.QTSOLIPROD, C_SOLICITA.DTUTILIZA, C_SOLICITA.TXOBSITMSOL, V_CDFORNECED, V_NOVADTENTREGA, V_CUSTOLISPREC, V_CDPERIENTR, V_NMPERIENTR,
               '1', 'T', NULL, NULL,
               NULL, NULL, C_SOLICITA.NMPRODUTO, V_DIAS, C_SOLICITA.CDORIGEMPORTAL, C_SOLICITA.NRORIGEMPORTAL, C_SOLICITA.NRITEMPORTAL);
        END IF;

        UPDATE ITEMSOLI
           SET USR_DTEXPORTA = SYSDATE, USR_NRSEQUENCIAL = V_NOVOCODIGO
         WHERE CDFILSOLI = C_SOLICITA.CDFILSOLI
           AND CDFILLAN = C_SOLICITA.CDFILLANCA
           AND NRSOLICMP = C_SOLICITA.NRSOLICMP
           AND CDPRODUTO = C_SOLICITA.CDPRODUTO
           AND DTPROGENTR = C_SOLICITA.DTPROGENTR;
        V_NRSEQ := LPAD(RTRIM(TO_CHAR(TO_NUMBER(RTRIM(V_NRSEQ))+ 1)),4,'0');
      END LOOP;
      IF V_NOVOCODIGO IS NOT NULL THEN
        UPDATE USR_INT_ITEMSOLI
           SET DTGRAVACAO = V_DTGRV, HRGRAVACAO = V_HRGRV
         WHERE USR_INT_ITEMSOLI.NRSEQUENCIAL = V_NOVOCODIGO;
         COMMIT;
      END IF;

      COMMIT;
      V_RETURN := USR_FUNC_BLOQUEIO('USR_INT_ITEMSOLI', 'D');
   END IF;
EXCEPTION
  WHEN OTHERS THEN BEGIN
     IF V_RETURN = 'S' THEN
        V_ERROR:= SQLERRM(SQLCODE);
        V_RETURN := USR_FUNC_BLOQUEIO('USR_INT_ITEMSOLI', 'D', V_ERROR);
     END IF;
  END;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('USR_EXPORTASOLICITA','5.09.xxx'); END;
/
